import { __spread, __values } from 'tslib';
import { Injectable, Directive, TemplateRef, ContentChild, Input, Component, ChangeDetectionStrategy, Output, EventEmitter, ElementRef, NgZone, ViewChild, ChangeDetectorRef, ViewEncapsulation, ContentChildren, ViewContainerRef, NgModule } from '@angular/core';
import { combineLatest, BehaviorSubject } from 'rxjs';
import { debounceTime } from 'rxjs/operators';
import { NgbModule } from '@ng-bootstrap/ng-bootstrap';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { ResizableModule } from 'angular-resizable-element';

var startCase = require('lodash/startCase');
var toLower = require('lodash/toLower');
var DataGridService = /** @class */ (function () {
    function DataGridService() {
    }
    DataGridService.prototype.mapPropertiesDown = function (array, mapObj) {
        return array.map(function (element) {
            for (var key in mapObj) {
                if (mapObj.hasOwnProperty(key)) {
                    if (!element[key]) {
                        element[key] = element[mapObj[key]];
                    }
                }
            }
            return element;
        });
    };
    DataGridService.prototype.mapPropertiesUp = function (array, mapObj) {
        return JSON.parse(JSON.stringify(array)).map(function (element) {
            for (var key in mapObj) {
                if (mapObj.hasOwnProperty(key)) {
                    element[mapObj[key]] = element[key];
                }
            }
            return element;
        });
    };
    DataGridService.prototype.mapColumns = function (columns) {
        var columnMap = {};
        columns.forEach(function (column) { return (columnMap[column.prop] = column); });
        return columnMap;
    };
    DataGridService.prototype.getVisibleRows = function (rows, scrollProps, rowsVisible, rowHeight) {
        var rowsNew = __spread(rows);
        var buffer = 5;
        if (window.navigator.userAgent.indexOf('Edge') > -1) {
            buffer = 15;
        }
        var offSetRowsFromTop = Math.floor(scrollProps.scrollTop / (rowHeight + 1));
        if (offSetRowsFromTop - buffer > 0) {
            offSetRowsFromTop -= buffer;
        }
        if (offSetRowsFromTop < buffer) {
            offSetRowsFromTop = 0;
        }
        var rowsEnd = offSetRowsFromTop + rowsVisible + buffer * 2;
        if (rowsEnd > rowsNew.length) {
            rowsEnd = rowsNew.length;
        }
        return rowsNew.slice(offSetRowsFromTop, rowsEnd);
    };
    DataGridService.prototype.getVisibleColumns = function (columns, scrollProps, gridProps) {
        var colsExternal = [];
        var buffer = 150;
        var widthCurrent = 0;
        for (var i = 0; i < columns.length; i++) {
            var column = columns[i];
            if (column.$$width + widthCurrent + buffer > scrollProps.scrollLeft &&
                widthCurrent < gridProps.widthViewPort + scrollProps.scrollLeft + buffer) {
                colsExternal.push(column);
            }
            widthCurrent = widthCurrent + column.$$width;
        }
        return __spread(colsExternal);
    };
    DataGridService.prototype.filterGlobal = function (array, filterGlobal) {
        var filters = [];
        filterGlobal.props.forEach(function (prop) {
            filters.push({
                operator: 'search',
                prop: prop,
                value: filterGlobal.term,
            });
        });
        return this.filterArray(array, filters);
    };
    DataGridService.prototype.filterArray = function (array, filters) {
        var contains = filters.filter(function (filter) { return filter.operator === 'contains'; });
        var searches = filters.filter(function (filter) { return filter.operator === 'search'; });
        var equals = {};
        filters.filter(function (filter) { return filter.operator === 'eq'; }).forEach(function (filter) {
            if (!equals[filter.prop]) {
                equals[filter.prop] = [];
            }
            equals[filter.prop].push(filter);
        });
        return array.filter(function (row) {
            if (searches.length) {
                var matches = false;
                for (var i = 0; i < searches.length; i++) {
                    var filter = searches[i];
                    if (!row[filter.prop]) {
                        return false;
                    }
                    var rowValue = row[filter.prop]
                        .toString()
                        .toLowerCase()
                        .trim();
                    var filterValue = filter.value
                        .toString()
                        .toLowerCase()
                        .trim();
                    if (rowValue.indexOf(filterValue) !== -1) {
                        matches = true;
                        break;
                    }
                }
                if (!matches) {
                    return false;
                }
            }
            if (contains.length) {
                var matches = false;
                for (var i = 0; i < contains.length; i++) {
                    var filter = contains[i];
                    if (!row[filter.prop]) {
                        return false;
                    }
                    var rowValue = row[filter.prop]
                        .toString()
                        .toLowerCase()
                        .trim();
                    var filterValue = filter.value
                        .toString()
                        .toLowerCase()
                        .trim();
                    if (rowValue.indexOf(filterValue) !== -1) {
                        matches = true;
                        break;
                    }
                }
                if (!matches) {
                    return false;
                }
            }
            if (Object.keys(equals).length) {
                var allGroupMatches = 0;
                for (var key in equals) {
                    if (equals.hasOwnProperty(key)) {
                        var matches = false;
                        for (var i = 0; i < equals[key].length; i++) {
                            var filter = equals[key][i];
                            if (!row[filter.prop] && row[filter.prop] !== false) {
                                return false;
                            }
                            if (Array.isArray(row[filter.prop]) &&
                                row[filter.prop] &&
                                row[filter.prop].indexOf(filter.value.toString()) > -1) {
                                matches = true;
                                break;
                            }
                            if (row[filter.prop] === filter.value) {
                                matches = true;
                                break;
                            }
                            if ((filter.value === 'True' && row[filter.prop] === true) ||
                                (filter.value === 'False' && row[filter.prop] === false)) {
                                matches = true;
                                break;
                            }
                        }
                        if (matches) {
                            allGroupMatches++;
                        }
                    }
                }
                if (allGroupMatches !== Object.keys(equals).length) {
                    return false;
                }
            }
            return true;
        });
    };
    DataGridService.prototype.sortArray = function (array, prop, sortType) {
        var mapProp = function (prop2) {
            if (typeof prop2 === 'string') {
                return prop2.toLowerCase().replace(/[^a-z0-9]/gi, '');
            }
            else if (typeof prop2 === 'number') {
                return prop2 * 1000;
            }
            else {
                if (prop2) {
                    return prop2.toString();
                }
                return '';
            }
        };
        if (sortType === 'asc') {
            return array.sort(function (a, b) { return (mapProp(a[prop]) !== mapProp(b[prop]) ? (mapProp(a[prop]) < mapProp(b[prop]) ? -1 : 1) : 0); });
        }
        else {
            return array.sort(function (b, a) { return (mapProp(a[prop]) !== mapProp(b[prop]) ? (mapProp(a[prop]) < mapProp(b[prop]) ? -1 : 1) : 0); });
        }
    };
    DataGridService.prototype.groupRows = function (rows, columns, groups, sorts, options) {
        var _this = this;
        var newGroups = {};
        var group = groups[0];
        rows.forEach(function (row) {
            var newProp = Array.isArray(row[group.prop]) ? row[group.prop].join() : row[group.prop];
            if (!newProp || newProp === '') {
                newProp = 'No Value';
            }
            var column = columns.filter(function (columnNew) { return columnNew.prop === group.prop; });
            if (!newGroups[newProp]) {
                newGroups[newProp] = ({
                    rows: [],
                    column: column && column[0] ? column[0] : null,
                    columnProp: group.prop,
                    columnLabel: '',
                    label: row[group.prop] || 'No Value',
                    visible: true,
                    type: 'group',
                });
                newGroups[newProp].columnLabel = column && column[0] ? column[0].label : 'No Value';
            }
            newGroups[newProp].rows.push(row);
        });
        var grouped = [];
        for (var key in newGroups) {
            if (newGroups.hasOwnProperty(key)) {
                grouped.push(newGroups[key]);
            }
        }
        grouped = this.sortArray(grouped, 'label', group.dir);
        var newRows = [];
        var groupsFinal = {};
        grouped.forEach(function (group2) {
            if (sorts.length) {
                _this.sortArray(group2.rows, sorts[0].prop, sorts[0].dir);
            }
            if (options.primaryKey) {
                ((group2))[options.primaryKey] = group2.label + '-' + group2.rows.length;
            }
            var currentLoc = newRows.length;
            groupsFinal[currentLoc] = group2;
            newRows = __spread(newRows, [group2], group2.rows);
        });
        return { rows: newRows, groups: groupsFinal };
    };
    DataGridService.prototype.createStatuses = function (state, columns) {
        var status = {
            groups: {},
            sorts: {},
            filters: {},
        };
        if (state.groups.length) {
            var newStatus_1 = {};
            state.groups.forEach(function (group) {
                newStatus_1[group.prop] = group.dir;
            });
            status.groups = newStatus_1;
        }
        else {
            status.groups = {};
        }
        if (state.sorts.length) {
            var newStatus_2 = {};
            state.sorts.forEach(function (sort) {
                newStatus_2[sort.prop] = sort.dir;
            });
            status.sorts = newStatus_2;
        }
        else {
            status.sorts = {};
        }
        var newFilters = {};
        if (columns && columns.length) {
            columns.forEach(function (column) {
                newFilters[column.prop] = {
                    contains: '',
                    eq: {},
                };
            });
        }
        if (state.filters && typeof state.filters !== 'string' && state.filters.length) {
            state.filters.forEach(function (filter) {
                if (!newFilters[filter.prop]) {
                    newFilters[filter.prop] = {
                        hasFilters: false,
                        contains: {},
                        eq: {},
                    };
                }
                if (filter.operator === 'contains') {
                    newFilters[filter.prop][filter.operator] = filter.value;
                    newFilters[filter.prop].hasFilters = true;
                }
                else {
                    if (!newFilters[filter.prop][filter.operator]) {
                        newFilters[filter.prop][filter.operator] = {};
                    }
                    newFilters[filter.prop][filter.operator][filter.value] = filter.value;
                    newFilters[filter.prop].hasFilters = true;
                }
            });
        }
        status.filters = newFilters;
        return status;
    };
    DataGridService.prototype.getDefaultTermsList = function (rows, columns) {
        var termsList = {};
        var uniques = {};
        columns.forEach(function (column) {
            if (!termsList[column.prop]) {
                termsList[column.prop] = [];
                uniques[column.prop] = {};
            }
        });
        rows.forEach(function (row) {
            for (var key in termsList) {
                if (termsList.hasOwnProperty(key)) {
                    if ((row[key] || row[key] === false) && uniques[key]) {
                        var keyNew = row[key];
                        if (typeof row[key] === 'boolean') {
                            keyNew = startCase(toLower(row[key].toString()));
                        }
                        uniques[key][keyNew] = true;
                    }
                }
            }
        });
        for (var key in uniques) {
            if (uniques.hasOwnProperty(key)) {
                var foo = uniques[key];
                for (var key2 in foo) {
                    if (foo.hasOwnProperty(key2)) {
                        termsList[key].push(key2);
                    }
                }
            }
        }
        for (var key in termsList) {
            if (termsList.hasOwnProperty(key)) {
                if (termsList[key][0] === 'False' || termsList[key][0] === 'True') {
                    termsList[key] = ['True', 'False'];
                }
                else {
                    termsList[key].sort();
                }
            }
        }
        return termsList;
    };
    DataGridService.prototype.columnCalculations = function (columns, offset) {
        if (offset === void 0) { offset = 0; }
        var leftOffset = offset;
        return columns.map(function (column, index) {
            var width = column.width ? column.width : 150;
            if (width < 44) {
                width = 44;
            }
            if (!column.width || !column.$$width) {
                column.width = width;
                column.$$width = width;
            }
            column.columnType = column.columnType ? column.columnType : 'string';
            column.$$index = index;
            column.$$leftOffset = leftOffset;
            leftOffset += width;
            return column;
        });
    };
    DataGridService.prototype.columnsResize = function (columns, widthColumns, widthTable) {
        var leftOffset = 0;
        var multiplier = Math.floor(widthTable / widthColumns * 100) / 100;
        return columns.map(function (column) {
            if (column.width) {
                column.$$width = Math.ceil(column.width * multiplier);
                column.$$leftOffset = leftOffset;
                leftOffset += column.$$width;
            }
            return Object.assign({}, column);
        });
    };
    DataGridService.prototype.templatesAddToColumns = function (columns, columnTemplates) {
        var templates = {};
        columns.forEach(function (column) {
            if (columnTemplates[column.prop]) {
                if (!templates[column.prop]) {
                    templates[column.prop] = {};
                }
                if (columnTemplates[column.prop].templateCell) {
                    templates[column.prop].templateCell = columnTemplates[column.prop].templateCell;
                }
                if (columnTemplates[column.prop].templateCell) {
                    templates[column.prop].templateHeader = columnTemplates[column.prop].templateHeader;
                }
            }
        });
        return templates;
    };
    DataGridService.prototype.templatesMapColumns = function (arr) {
        var result = {};
        try {
            for (var arr_1 = __values(arr), arr_1_1 = arr_1.next(); !arr_1_1.done; arr_1_1 = arr_1.next()) {
                var temp = arr_1_1.value;
                var col = {};
                var props = Object.getOwnPropertyNames(temp);
                try {
                    for (var props_1 = __values(props), props_1_1 = props_1.next(); !props_1_1.done; props_1_1 = props_1.next()) {
                        var prop = props_1_1.value;
                        col[prop] = temp[prop];
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (props_1_1 && !props_1_1.done && (_a = props_1.return)) _a.call(props_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                if (temp.headerTemplate) {
                    col.templateHeader = temp.templateHeader;
                }
                if (temp.cellTemplate) {
                    col.templateCell = temp.templateCell;
                }
                result[col.prop] = col;
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (arr_1_1 && !arr_1_1.done && (_b = arr_1.return)) _b.call(arr_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return result;
        var e_2, _b, e_1, _a;
    };
    DataGridService.prototype.rowPositions = function (rows, rowHeight, makeVisible) {
        if (makeVisible === void 0) { makeVisible = false; }
        var y = 0;
        return rows.filter(function (row, i) {
            row.$$rowIndex = i;
            if (typeof row.$$hidden === 'undefined' || makeVisible) {
                row.$$hidden = false;
            }
            if (row.$$hidden === false) {
                row.$$rowPosition = y;
                y += rowHeight + 1;
                return true;
            }
            else {
                return false;
            }
        });
    };
    return DataGridService;
}());
DataGridService.decorators = [
    { type: Injectable },
];
DataGridService.ctorParameters = function () { return []; };
var DataGridColumnHeaderDirective = /** @class */ (function () {
    function DataGridColumnHeaderDirective(template) {
        this.template = template;
    }
    return DataGridColumnHeaderDirective;
}());
DataGridColumnHeaderDirective.decorators = [
    { type: Directive, args: [{ selector: '[datagrid-header-template]' },] },
];
DataGridColumnHeaderDirective.ctorParameters = function () { return [
    { type: TemplateRef, },
]; };
var DataGridColumnCellDirective = /** @class */ (function () {
    function DataGridColumnCellDirective(template) {
        this.template = template;
    }
    return DataGridColumnCellDirective;
}());
DataGridColumnCellDirective.decorators = [
    { type: Directive, args: [{ selector: '[datagrid-cell-template]' },] },
];
DataGridColumnCellDirective.ctorParameters = function () { return [
    { type: TemplateRef, },
]; };
var DataTableColumnDirective = /** @class */ (function () {
    function DataTableColumnDirective() {
    }
    return DataTableColumnDirective;
}());
DataTableColumnDirective.decorators = [
    { type: Directive, args: [{ selector: 'datagrid-column' },] },
];
DataTableColumnDirective.propDecorators = {
    "name": [{ type: Input },],
    "prop": [{ type: Input },],
    "templateCell": [{ type: Input }, { type: ContentChild, args: [DataGridColumnCellDirective, { read: TemplateRef },] },],
    "templateHeader": [{ type: Input }, { type: ContentChild, args: [DataGridColumnHeaderDirective, { read: TemplateRef },] },],
};
var Actions = {
    group: 'group',
    sort: 'sort',
    pinLeft: 'pinLeft',
    filter: 'filter',
    reset: 'reset',
    column: 'column',
};
var throttle = require('lodash/throttle');
var BodyComponent = /** @class */ (function () {
    function BodyComponent(zone, element) {
        var _this = this;
        this.zone = zone;
        this.onColumnsUpdated = new EventEmitter();
        this.onStateUpdated = new EventEmitter();
        this.onCustomLinkEvent = new EventEmitter();
        this.onRowUpdated = new EventEmitter();
        this.onRowMouseEvent = new EventEmitter();
        this.onGroupToggled = new EventEmitter();
        this.onRightClick = new EventEmitter();
        this.onRowMouseDown = new EventEmitter();
        this.onRowMouseUp = new EventEmitter();
        this.onScrollEvent = new EventEmitter();
        this.onScrollThrottled = throttle(function (event) { return _this.onScroll(event); }, 20, { trailing: true, leading: true });
        this.body = element.nativeElement;
    }
    BodyComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.zone.runOutsideAngular(function () {
            _this.body.addEventListener('scroll', _this.onScrollThrottled.bind(_this), ({ capture: true, passive: true }));
        });
    };
    BodyComponent.prototype.onScroll = function (event) {
        var _this = this;
        requestAnimationFrame(function () {
            var scrollProps = {
                scrollTop: event.target.scrollTop,
                scrollLeft: event.target.scrollLeft,
            };
            _this.onScrollEvent.emit(scrollProps);
        });
    };
    BodyComponent.prototype.trackRow = function (_index, item) {
        return item.$$track;
    };
    BodyComponent.prototype.onMouseEvent = function (type, rowIndex, event) {
        this.onRowMouseEvent.emit({ type: type, rowIndex: rowIndex, event: event });
    };
    BodyComponent.prototype.rowUpdated = function (event, rowIndex) {
        event.rowIndex = rowIndex;
        this.onRowUpdated.emit(event);
    };
    BodyComponent.prototype.groupToggled = function (group) {
        this.onGroupToggled.emit(group);
    };
    BodyComponent.prototype.ngOnDestroy = function () {
        this.body.removeEventListener('scroll', this.onScrollThrottled.bind(this), ({ capture: true, passive: true }));
    };
    return BodyComponent;
}());
BodyComponent.decorators = [
    { type: Component, args: [{
                selector: 'datagrid-body',
                template: "<div class=\"datagrid-body-container\" id=\"dataGridBody\" #dataGridBody\n\t [ngStyle]=\"{'height': gridProps.heightTotal + 'px'}\"\n\t [ngClass]=\"{ 'widthFixed' : gridProps.widthFixed }\">\n\n\t<div class=\"datagrid-body\" [ngStyle]=\"{'width' : gridProps.widthViewPort + 'px', 'height' : gridProps.heightBody + 'px' }\">\n\n\t\t<ng-container *ngFor=\"let row of rows; let rowIndex = index; trackBy:trackRow;\">\n\n\t\t\t<div class=\"datagrid-body-row\" *ngIf=\"!row.$$hidden\"\n\t\t\t\t [style.top]=\"row.$$rowPosition + 'px'\"\n\t\t\t\t [ngClass]=\"{'selected': row.$$selected }\">\n\n\t\t\t\t<!-- Group header. Treated as a row -->\n\t\t\t\t<group-header *ngIf=\"row.type == 'group' \"\n\t\t\t\t\t\t\t  [group]=\"row\"\n\t\t\t\t\t\t\t  [options]=\"options\"\n\t\t\t\t\t\t\t  [width]=\"gridProps.widthTotal\"\n\t\t\t\t\t\t\t  (onGroupToggled)=\"groupToggled($event)\">\n\t\t\t\t</group-header>\n\n\t\t\t\t<!-- Normal Row -->\n\t\t\t\t<ng-container *ngIf=\"row.type != 'group' \">\n\n\t\t\t\t\t<!-- Pinned columns -->\n\t\t\t\t\t<div class=\"datagrid-body-pinned sticky-left\" *ngIf=\"columnsPinnedLeft?.length\"\n\t\t\t\t\t\t [style.width]=\"gridProps.widthPinned + 'px'\"\n\t\t\t\t\t\t [ngStyle]=\"rowStyles[row[options.primaryKey]]\"\n\t\t\t\t\t\t (click)=\"onMouseEvent('click', row.$$rowIndex, $event)\"\n\t\t\t\t\t\t (dblclick)=\"onMouseEvent('dblclick', row.$$rowIndex, $event)\"\n\t\t\t\t\t\t (contextmenu)=\"onMouseEvent('contextmenu', row.$$rowIndex, $event)\"\n\t\t\t\t\t\t (mousedown)=\"onMouseEvent('mousedown', row.$$rowIndex, $event)\"\n\t\t\t\t\t\t (mouseup)=\"onMouseEvent('mouseup', row.$$rowIndex, $event)\"\n\t\t\t\t\t\t (mouseenter)=\"onMouseEvent('mouseenter', row.$$rowIndex, $event)\">\n\t\t\t\t\t\t<datagrid-body-row [columns]=\"columnsPinnedLeft\"\n\t\t\t\t\t\t\t\t\t\t   [row]=\"row\"\n\t\t\t\t\t\t\t\t\t\t   [options]=\"options\"\n                       [templates]=\"templates\"\n\t\t\t\t\t\t\t\t\t\t   (onRowUpdated)=\"rowUpdated($event, row.$$rowIndex)\">\n\t\t\t\t\t\t</datagrid-body-row>\n\t\t\t\t\t</div>\n\n\t\t\t\t\t<!-- Non-pinned columns -->\n\t\t\t\t\t<div class=\"datagrid-body-main\" style=\"display: inline-block\"\n\t\t\t\t\t\t [style.width]=\"gridProps.widthMain + 'px'\"\n\t\t\t\t\t\t [ngStyle]=\"rowStyles[row[options.primaryKey]]\"\n\t\t\t\t\t\t (click)=\"onMouseEvent('click', row.$$rowIndex, $event)\"\n\t\t\t\t\t\t (dblclick)=\"onMouseEvent('dblclick', row.$$rowIndex, $event)\"\n\t\t\t\t\t\t (contextmenu)=\"onMouseEvent('contextmenu', row.$$rowIndex, $event)\"\n\t\t\t\t\t\t (mousedown)=\"onMouseEvent('mousedown', row.$$rowIndex, $event)\"\n\t\t\t\t\t\t (mouseup)=\"onMouseEvent('mouseup', row.$$rowIndex, $event)\"\n\t\t\t\t\t\t (mouseenter)=\"onMouseEvent('mouseenter', row.$$rowIndex, $event)\">\n\t\t\t\t\t\t<datagrid-body-row [columns]=\"columns\"\n\t\t\t\t\t\t\t\t\t\t   [row]=\"row\"\n\t\t\t\t\t\t\t\t\t\t   [options]=\"options\"\n                       [templates]=\"templates\"\n\t\t\t\t\t\t\t\t\t\t   (onRowUpdated)=\"rowUpdated($event, row.$$rowIndex)\">\n\t\t\t\t\t\t</datagrid-body-row>\n\t\t\t\t\t</div>\n\n\t\t\t\t</ng-container>\n\n\t\t\t</div>\n\t\t</ng-container>\n\t</div>\n</div>\n",
                styles: [],
                changeDetection: ChangeDetectionStrategy.OnPush,
            },] },
];
BodyComponent.ctorParameters = function () { return [
    { type: NgZone, },
    { type: ElementRef, },
]; };
BodyComponent.propDecorators = {
    "columns": [{ type: Input },],
    "columnsPinnedLeft": [{ type: Input },],
    "rows": [{ type: Input },],
    "rowStyles": [{ type: Input },],
    "state": [{ type: Input },],
    "status": [{ type: Input },],
    "options": [{ type: Input },],
    "gridProps": [{ type: Input },],
    "templates": [{ type: Input },],
    "onColumnsUpdated": [{ type: Output },],
    "onStateUpdated": [{ type: Output },],
    "onCustomLinkEvent": [{ type: Output },],
    "onRowUpdated": [{ type: Output },],
    "onRowMouseEvent": [{ type: Output },],
    "onGroupToggled": [{ type: Output },],
    "onRightClick": [{ type: Output },],
    "onRowMouseDown": [{ type: Output },],
    "onRowMouseUp": [{ type: Output },],
    "onScrollEvent": [{ type: Output },],
};
var throttle$1 = require('lodash/throttle');
var DataGridComponent = /** @class */ (function () {
    function DataGridComponent(dgSvc, ref) {
        var _this = this;
        this.dgSvc = dgSvc;
        this.ref = ref;
        this._state = {
            filters: [],
            sorts: [],
            groups: [],
            info: {},
        };
        this.onColumnsUpdated = new EventEmitter();
        this.onRowsSelected = new EventEmitter();
        this.onStateChange = new EventEmitter();
        this.onRightClickMenu = new EventEmitter();
        this.action = new EventEmitter();
        this.onCustomLinkEvent = new EventEmitter();
        this.onElementRef = new EventEmitter();
        this.onRowUpdated = new EventEmitter();
        this.columnsPinnedLeft = [];
        this.columnsPinnedRight = [];
        this.gridProps = {
            widthTotal: 0,
            widthPinned: 0,
            widthMain: 0,
            heightTotal: 0,
            heightBody: 0,
            widthViewPort: 0,
            widthFixed: true,
        };
        this.scrollProps = { scrollTop: 0, scrollLeft: 0 };
        this.templatesCell = {};
        this.columnsMapped = {};
        this.rowClasses = {};
        this.rowStyles = {};
        this.appReady = false;
        this.domReady = false;
        this.dragging = false;
        this.draggingPos = {
            hasMinSize: false,
            startX: 0,
            startY: 0,
            top: 0,
            bottom: 0,
            left: 0,
            right: 0,
            width: 0,
            height: 0,
            bounding: {
                top: 0,
                bottom: 0,
                left: 0,
                right: 0,
            },
        };
        this.keysPressed = {};
        this.rowClickDrag = {
            rowIndex: 0,
            groupIndex: 0,
        };
        this.columnWidthsInternal = 0;
        this.rowHeight = 23;
        this.rowsIndexes = { start: 0, end: 0 };
        this.columnIndexes = { start: 0, end: 0 };
        this.onWindowResizeThrottled = throttle$1(function () { return _this.onWindowResize(); }, 300, { trailing: true, leading: true });
        this.subscriptions = [];
        this.rowGroups = [];
        this.groups = {};
    }
    Object.defineProperty(DataGridComponent.prototype, "columns", {
        get: function () {
            return this._columns && this._columns.length ? __spread(this._columns) : [];
        },
        set: function (columns) {
            if (columns && columns.length) {
                var slug_1 = Math.floor(Math.random() * 1000000);
                columns = columns.map(function (column, i) {
                    column.$$track = slug_1 + '-' + i;
                    return Object.assign({}, column);
                });
                if (this.options && this.options.columnMap) {
                    columns = this.dgSvc.mapPropertiesDown(columns, this.options.columnMap);
                }
            }
            this._columns = columns;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataGridComponent.prototype, "rows", {
        get: function () {
            return this._rows && this._rows.length ? __spread(this._rows) : [];
        },
        set: function (rows) {
            if (rows && rows.length) {
                var slug_2 = Math.floor(Math.random() * 1000000);
                rows.forEach(function (row, i) {
                    row.$$track = slug_2 + '-' + i;
                    row.$$selected = false;
                });
            }
            this._rows = rows;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataGridComponent.prototype, "state", {
        get: function () {
            return this._state;
        },
        set: function (state) {
            var stateNew = state ? state : {};
            if (!stateNew.filters) {
                stateNew.filters = [];
            }
            if (!stateNew.sorts) {
                stateNew.sorts = [];
            }
            if (!stateNew.groups) {
                stateNew.groups = [];
            }
            if (!stateNew.info) {
                stateNew.info = {};
            }
            stateNew.groups =
                stateNew.groups && stateNew.groups.length && this.options.controlsMap
                    ? this.dgSvc.mapPropertiesDown(stateNew.groups, this.options.controlsMap)
                    : stateNew.groups;
            stateNew.sorts =
                stateNew.sorts && stateNew.sorts.length && this.options.controlsMap
                    ? this.dgSvc.mapPropertiesDown(stateNew.sorts, this.options.controlsMap)
                    : stateNew.sorts;
            stateNew.filters =
                stateNew.filters && stateNew.filters.length && this.options.controlsMap
                    ? this.dgSvc.mapPropertiesDown(stateNew.filters, this.options.controlsMap)
                    : stateNew.filters;
            stateNew.info = {
                initial: true,
            };
            this._state = stateNew;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataGridComponent.prototype, "columnTemplates", {
        get: function () {
            return this._columnTemplates;
        },
        set: function (val) {
            var arr = val.toArray();
            if (arr.length) {
                this._columnTemplates = this.dgSvc.templatesMapColumns(arr);
            }
        },
        enumerable: true,
        configurable: true
    });
    DataGridComponent.prototype.ngOnInit = function () { };
    DataGridComponent.prototype.ngOnChanges = function (model) {
        var _this = this;
        if (model.filterGlobal && this.filterGlobal && this.filterGlobal.term) {
            throttle$1(function () { return _this.viewCreate(); }, 500, { trailing: true, leading: true });
        }
        if (model.columns) {
            var columns = this.options.columnMap
                ? this.dgSvc.mapPropertiesDown(this.columns, this.options.columnMap)
                : this.columns;
            var columnsPinnedLeft = columns.filter(function (column) { return (column.pinnedLeft ? true : false); });
            this.columnsPinnedLeft = columns.length ? this.dgSvc.columnCalculations(columnsPinnedLeft) : [];
            var columnsInternal = columns.filter(function (column) { return (!column.pinnedLeft ? true : false); });
            this.columnsInternal = columns.length ? this.dgSvc.columnCalculations(columnsInternal) : [];
            this.columnWidthsInternal = this.columnsInternal.reduce(function (a, b) { return (b.width ? a + b.width : 0); }, 0);
            this.columnsMapped = this.dgSvc.mapColumns(this.columns);
        }
        if (this.state && this.columns) {
            if (this.state.sorts &&
                this.state.sorts.length &&
                this.state.sorts[0].prop &&
                !this.columnsMapped[this.state.sorts[0].prop]) {
                this.state.sorts = [];
                console.error("Sorting option is for a column that doesn't exist. Sort option has been removed.");
            }
            if (this.state.groups && this.state.groups.length && !this.columnsMapped[this.state.groups[0].prop]) {
                this.state.groups = [];
                console.error("Grouping option is for a column that doesn't exist. Group option has been removed.");
            }
            if (this.state.filters && this.state.filters.length) {
                var _loop_1 = function (i) {
                    if (!this_1.columnsMapped[this_1.state.filters[i].prop]) {
                        this_1.state.filters = this_1.state.filters.filter(function (_filter, index) { return i !== index; });
                        console.error("Filter option is for a column that doesn't exist. Filter option has been removed.");
                    }
                };
                var this_1 = this;
                for (var i = this.state.filters.length - 1; i >= 0; i--) {
                    _loop_1(i);
                }
            }
        }
        if (this.columns && this.rows) {
            this.rows.forEach(function (row) { return (row.$$selected = false); });
            this.filterTerms = this.dgSvc.getDefaultTermsList(this.rows, this.columns);
            this.createRowStyles();
            this.createRowClasses();
            this.appReady = true;
            this.dataGridReady();
            this.emitState(this.state);
        }
    };
    DataGridComponent.prototype.ngAfterViewInit = function () {
        this.domReady = true;
        this.dataGridReady();
    };
    DataGridComponent.prototype.dataGridReady = function () {
        var _this = this;
        if (this.appReady && this.domReady) {
            if (this.dataGrid && this.dataGrid.nativeElement) {
                this.gridProps.widthViewPort = Math.floor(this.dataGrid.nativeElement.getBoundingClientRect().width);
            }
            if (this.gridProps.widthViewPort) {
                this.viewCreate();
            }
            else {
                setTimeout(function () {
                    _this.dataGridReady();
                }, 50);
            }
        }
    };
    DataGridComponent.prototype.onScroll = function (scrollPropsNew) {
        this.ref.detach();
        var scrollPropsOld = Object.assign({}, this.scrollProps);
        var scrollProps = Object.assign({}, scrollPropsNew);
        if (scrollPropsOld.scrollTop !== scrollProps.scrollTop) {
            var rowsVisible = Math.ceil(this.gridProps.heightTotal / this.rowHeight);
            var rowsExternal = this.dgSvc.getVisibleRows(this.rowsInternal, scrollProps, rowsVisible, this.rowHeight);
            if (!this.rowsIndexes ||
                (rowsExternal[0] &&
                    rowsExternal[0].$$track !== this.rowsIndexes.start &&
                    rowsExternal[rowsExternal.length - 1].$$track !== this.rowsIndexes.end)) {
                this.rowsExternal = rowsExternal;
                this.rowsIndexes = {
                    start: rowsExternal[0].$$track,
                    end: rowsExternal[rowsExternal.length - 1].$$track,
                };
            }
        }
        if (scrollPropsOld.scrollLeft !== scrollProps.scrollLeft) {
            var columnsExternal = this.dgSvc.getVisibleColumns(this.columnsInternal, scrollProps, this.gridProps);
            if (!this.columnIndexes ||
                (columnsExternal[0] &&
                    columnsExternal[0].$$track !== this.columnIndexes.start &&
                    columnsExternal[columnsExternal.length - 1].$$track !== this.columnIndexes.end)) {
                this.columnsExternal = columnsExternal;
                this.columnIndexes = {
                    start: columnsExternal[0].$$track,
                    end: columnsExternal[columnsExternal.length - 1].$$track,
                };
            }
        }
        this.scrollProps = scrollProps;
        this.ref.reattach();
        this.ref.detectChanges();
    };
    DataGridComponent.prototype.viewCreate = function () {
        this.ref.detach();
        var newRows = this.rows;
        if (this.filterGlobal && this.filterGlobal.term) {
            newRows = this.dgSvc.filterGlobal(newRows, this.filterGlobal);
        }
        if (this.state && this.state.filters && this.state.filters.length) {
            newRows = this.dgSvc.filterArray(newRows, this.state.filters);
        }
        if (this.state && this.state.groups && this.state.sorts && this.state.groups.length) {
            var groupings = this.dgSvc.groupRows(newRows, this.columns, this.state.groups, this.state.sorts, this.options);
            newRows = groupings.rows;
            this.groups = groupings.groups;
        }
        else {
            this.groups = null;
            if (this.state &&
                this.state.sorts &&
                this.state.sorts.length &&
                this.state.sorts[0] &&
                this.state.sorts[0].prop &&
                this.state.sorts !== undefined) {
                newRows = this.dgSvc.sortArray(newRows, this.state.sorts[0].prop, this.state.sorts[0].dir);
            }
        }
        newRows = this.dgSvc.rowPositions(newRows, this.rowHeight);
        this.updateGridProps();
        this.columnsPinnedLeft = this.columnsPinnedLeft.length ? this.dgSvc.columnCalculations(this.columnsPinnedLeft) : [];
        this.columnsInternal = this.columnsInternal.length ? this.dgSvc.columnCalculations(this.columnsInternal) : [];
        if (this.dataGrid &&
            this.dataGrid.nativeElement &&
            this.columnWidthsInternal < this.dataGrid.nativeElement.getBoundingClientRect().width) {
            this.columnsInternal = this.dgSvc.columnsResize(this.columnsInternal, this.columnWidthsInternal, this.gridProps.widthViewPort - this.gridProps.widthPinned);
            this.gridProps.widthFixed = true;
        }
        else {
            this.columnsInternal = this.columnsInternal.map(function (column) {
                column.$$width = column.width;
                return column;
            });
            this.gridProps.widthFixed = false;
        }
        this.updateGridProps();
        this.rowsInternal = newRows;
        this.columnsExternal = this.dgSvc.getVisibleColumns(this.columnsInternal, this.scrollProps, this.gridProps);
        var rowsVisible = Math.ceil(this.gridProps.heightTotal / this.rowHeight);
        this.rowsExternal = this.dgSvc.getVisibleRows(this.rowsInternal, this.scrollProps, rowsVisible, this.rowHeight);
        if (this.state.info) {
            this.state.info.rowsTotal = this.rows.length;
            this.state.info.rowsVisible = this.rowsInternal.filter(function (row) { return !row.type; }).length;
        }
        this.status = this.dgSvc.createStatuses(this.state, this.columnsInternal);
        this.state = Object.assign({}, this.state);
        this.emitState(this.state);
        this.ref.reattach();
    };
    DataGridComponent.prototype.groupToggled = function () {
        this.viewCreate();
    };
    DataGridComponent.prototype.onStateUpdated = function (stateChange) {
        this.ref.detach();
        var newState = Object.assign({}, this.state);
        if (!newState.filters || !Array.isArray(newState.filters)) {
            newState.filters = [];
        }
        if (!newState.sorts) {
            newState.sorts = [];
        }
        if (!newState.groups) {
            newState.groups = [];
        }
        if (!newState.info) {
            newState.info = {};
        }
        newState.info.initial = false;
        if (stateChange.action === Actions.sort) {
            newState.sorts = stateChange.data.dir ? [stateChange.data] : [];
        }
        else if (stateChange.action === Actions.group) {
            newState.groups = stateChange.data.dir ? [stateChange.data] : [];
        }
        else if (stateChange.action === Actions.filter) {
            var newFilter = stateChange.data.filter;
            if (stateChange.data.filterAction === 'change') {
                var index = 0;
                for (var i = 0; i < newState.filters.length; i++) {
                    if (newFilter.prop === newState.filters[i].prop && newFilter.operator === newState.filters[i].operator) {
                        index = i;
                        break;
                    }
                }
                newState.filters[index] = newFilter;
            }
            else if (stateChange.data.filterAction === 'add') {
                newState.filters.push(newFilter);
            }
            else if (stateChange.data.filterAction === 'remove') {
                var index = 0;
                for (var i = 0; i < newState.filters.length; i++) {
                    if (newFilter.operator === 'contains' &&
                        newFilter.prop === newState.filters[i].prop &&
                        newFilter.operator === newState.filters[i].operator) {
                        index = i;
                        break;
                    }
                    else if (newFilter.prop === newState.filters[i].prop &&
                        newFilter.operator === newState.filters[i].operator &&
                        newFilter.value === newState.filters[i].value) {
                        index = i;
                        break;
                    }
                }
                newState.filters.splice(index, 1);
            }
            else if (stateChange.data.filterAction === 'clear') {
                newState.filters = newState.filters.filter(function (item) { return item.prop !== stateChange.data.filter.prop; });
            }
        }
        else if (stateChange.action === Actions.reset) {
        }
        else if (stateChange.action === Actions.column) {
            if (stateChange.data.action === 'delete') {
                this.columnsInternal = this.columnsInternal.filter(function (column) { return column.$$index !== stateChange.data.columnIndex; });
                this.columns = this.columns.filter(function (column) { return column.$$index !== stateChange.data.columnIndex; });
                this.columnWidthsInternal = this.columnsInternal.reduce(function (a, b) { return (b.width ? a + b.width : 0); }, 0);
                this.emitColumns();
            }
        }
        else if (stateChange.action === Actions.pinLeft) {
            if (stateChange.data.isPinned) {
                var colNew = this.columnsPinnedLeft[stateChange.data.index];
                delete colNew.pinnedLeft;
                this.columnsPinnedLeft = this.columnsPinnedLeft.filter(function (_col, index) { return stateChange.data.index !== index; });
                this.columnsInternal = __spread([colNew], this.columnsInternal);
            }
            else {
                var newCol = this.columnsInternal.filter(function (col) { return col.prop === stateChange.data.prop; })[0];
                newCol.pinnedLeft = true;
                this.columnsPinnedLeft = __spread(this.columnsPinnedLeft, [newCol]);
                this.columnsInternal = this.columnsInternal.filter(function (col) { return col.prop !== stateChange.data.prop; });
            }
            this.columnWidthsInternal = this.columnsInternal.reduce(function (a, b) { return (b.width ? a + b.width : 0); }, 0);
            this.emitColumns();
        }
        this.state = newState;
        this.viewCreate();
    };
    DataGridComponent.prototype.columnsUpdated = function (columnData) {
        if (columnData && columnData.columnIndex) {
            if (columnData.action === 'resize') {
                if (columnData.type === 'pinnedLeft') {
                    this.columnsPinnedLeft[columnData.columnIndex].width = columnData.width;
                    this.columnsPinnedLeft[columnData.columnIndex] = Object.assign({}, this.columnsPinnedLeft[columnData.columnIndex]);
                }
                else {
                    this.columnsInternal[columnData.columnIndex].width = columnData.width;
                    this.columnsInternal[columnData.columnIndex] = Object.assign({}, this.columnsInternal[columnData.columnIndex]);
                }
            }
            else if (columnData.action === 'reorder') {
                if (columnData.type === 'pinnedLeft') {
                    var colOld = this.columnsPinnedLeft.filter(function (column) { return column.prop === columnData.prop; })[0];
                    var colsNew = this.columnsPinnedLeft.filter(function (column) { return column.prop !== columnData.prop; });
                    colsNew.splice(columnData.columnIndex, 0, colOld);
                    this.columnsPinnedLeft = colsNew;
                }
                else {
                    var colOld = this.columnsInternal.filter(function (column) { return column.prop === columnData.prop; })[0];
                    var colsNew = this.columnsInternal.filter(function (column) { return column.prop !== columnData.prop; });
                    colsNew.splice(columnData.columnIndex, 0, colOld);
                    this.columnsInternal = colsNew;
                }
            }
            this.columnWidthsInternal = this.columnsInternal.reduce(function (a, b) { return (b.width ? a + b.width : 0); }, 0);
            this.emitColumns();
            this.viewCreate();
        }
    };
    DataGridComponent.prototype.updateGridProps = function () {
        var gridProps = Object.assign({}, this.gridProps);
        if (this.dataGrid && this.dataGrid.nativeElement.getBoundingClientRect().width) {
            gridProps.widthViewPort = Math.floor(this.dataGrid.nativeElement.getBoundingClientRect().width) || 0;
        }
        gridProps.widthPinned = this.columnsPinnedLeft.length
            ? this.columnsPinnedLeft.reduce(function (a, b) { return (b.$$width ? a + b.$$width : 0); }, 0)
            : 0;
        gridProps.widthMain = this.columnsInternal.reduce(function (a, b) { return (b.$$width ? a + b.$$width : 0); }, 0) || 0;
        gridProps.widthTotal = gridProps.widthMain + gridProps.widthPinned;
        if (this.options.heightMax) {
            gridProps.heightTotal = (this.options.heightMax);
        }
        else if (this.options.fullScreen) {
            var height = this.dataGrid.nativeElement.getBoundingClientRect().height;
            var newHeight = height - 2 - this.rowHeight;
            if (this.options.showInfo && (this.state.sorts.length || this.state.groups.length || this.state.filters.length)) {
                newHeight -= this.rowHeight;
            }
            gridProps.heightTotal = newHeight;
        }
        else {
            gridProps.heightTotal = 300;
        }
        if (this.rowsInternal && this.rowsInternal.length) {
            gridProps.heightBody = this.rowsInternal.length * this.rowHeight;
        }
        else if (this.rows && this.rows.length) {
            gridProps.heightBody = this.rows.length * this.rowHeight;
        }
        else {
            gridProps.heightBody = 300;
        }
        this.gridProps = gridProps;
    };
    DataGridComponent.prototype.handleMouseDown = function (event) {
        var draggingPos = Object.assign({ startX: event.pageX, startY: event.pageY, bounding: this.dataGridBody.body.getBoundingClientRect() }, this.draggingPos);
        if (event.which === 1 &&
            draggingPos.startY > draggingPos.bounding.top &&
            draggingPos.startY < draggingPos.bounding.bottom &&
            draggingPos.startX > draggingPos.bounding.left &&
            draggingPos.startX < draggingPos.bounding.right) {
            this.draggingPos = draggingPos;
            this.dragging = true;
        }
    };
    DataGridComponent.prototype.handleMouseUp = function (event) {
        if (this.dragging && this.draggingPos && this.draggingPos.bounding && event.pageY < this.draggingPos.bounding.top) {
            this.rowHoveredLast = 0;
        }
        if (this.dragging && this.rowHoveredLast !== null) {
            this.onRowMouseEvent({ type: 'mouseup', rowIndex: this.rowHoveredLast, event: event });
            if (document.getSelection) {
                document.getSelection().removeAllRanges();
            }
            else if (window.getSelection) {
                window.getSelection().removeAllRanges();
            }
        }
        this.dragging = false;
    };
    DataGridComponent.prototype.handleMouseMove = function (event) {
        var draggingPos = Object.assign({ hasMinSize: false }, this.draggingPos);
        if (draggingPos &&
            draggingPos.bounding &&
            draggingPos.bounding.top &&
            draggingPos.bounding.bottom &&
            draggingPos.bounding.left &&
            draggingPos.bounding.right &&
            draggingPos.height &&
            draggingPos.top &&
            draggingPos.startY &&
            draggingPos.startX) {
            var pageY = event.pageY;
            var pageX = event.pageX;
            if (pageY < draggingPos.bounding.top) {
                pageY = draggingPos.bounding.top;
            }
            if (pageY > draggingPos.bounding.bottom) {
                pageY = draggingPos.bounding.bottom;
            }
            if (pageX < draggingPos.bounding.left) {
                pageX = draggingPos.bounding.left;
            }
            if (pageX > draggingPos.bounding.right) {
                pageX = draggingPos.bounding.right;
            }
            if (pageX >= draggingPos.startX) {
                draggingPos.width = pageX - draggingPos.startX - 2;
                draggingPos.left = draggingPos.startX;
            }
            else {
                draggingPos.width = draggingPos.startX - pageX;
                draggingPos.left = pageX + 2;
            }
            if (pageX < draggingPos.bounding.right && pageX > draggingPos.bounding.left) {
                if (pageY >= draggingPos.startY) {
                    draggingPos.height = pageY - draggingPos.startY - 2;
                    draggingPos.top = draggingPos.startY;
                }
                else {
                    draggingPos.height = draggingPos.startY - pageY;
                    draggingPos.top = pageY + 2;
                }
            }
            if (draggingPos.width > 10 && draggingPos.height > 10) {
                draggingPos.hasMinSize = true;
            }
            if (draggingPos.top < draggingPos.bounding.top) {
                draggingPos.top = draggingPos.bounding.top;
            }
            if (draggingPos.height > draggingPos.bounding.top) {
            }
            this.draggingPos = draggingPos;
        }
    };
    DataGridComponent.prototype.handleKeyboardEvents = function (event) {
        if (event.repeat === false) {
            this.keyPressed = event.type === 'keydown' ? event.key : null;
            if (event.type === 'keydown') {
                this.keysPressed[event.key.toString().toLowerCase()] = true;
            }
            else if (event.type === 'keyup') {
                delete this.keysPressed[event.key.toString().toLowerCase()];
            }
            if (this.keysPressed['a'] && this.keysPressed['Control']) {
                this.selectRowsAll();
                event.preventDefault();
                event.stopPropagation();
            }
        }
    };
    DataGridComponent.prototype.selectRowsAll = function () {
        if (this.rows.length) {
            this.rowsInternal.forEach(function (row) { return (row.$$selected = true); });
            this.emitSelectedRows(this.rowsInternal);
        }
    };
    DataGridComponent.prototype.onRowMouseEvent = function (action) {
        if (action.type !== 'mouseenter') {
        }
        var row = this.rowsInternal.filter(function (row2) { return row2.$$rowIndex === action.rowIndex; })[0];
        switch (action.type) {
            case 'click':
                this.selectRow(row, action.rowIndex, false);
                break;
            case 'contextmenu':
                this.selectRow(row, action.rowIndex, true);
                this.onRightClickMenu.emit(event);
                break;
            case 'mousedown':
                this.handleMouseDown(action.event);
                if (action.event.which === 1) {
                    this.rowClickDrag.rowIndex = action.rowIndex;
                }
                break;
            case 'mouseup':
                if (action.event.which === 1 && action.rowIndex !== this.rowClickDrag.rowIndex) {
                    var rowStart = this.rowClickDrag.rowIndex <= action.rowIndex ? this.rowClickDrag.rowIndex : action.rowIndex;
                    var rowEnd = this.rowClickDrag.rowIndex >= action.rowIndex ? this.rowClickDrag.rowIndex : action.rowIndex;
                    this.rowsInternal.forEach(function (rowNew) { return (rowNew.$$selected = false); });
                    for (var j = rowStart; j <= rowEnd; j++) {
                        this.rowsInternal[j].$$selected = true;
                    }
                    var selectedRows = this.rowsInternal.filter(function (rowNew) { return rowNew.$$selected; });
                    this.rowsSelectedCount = selectedRows.length;
                    this.emitSelectedRows(selectedRows);
                }
                break;
            case 'mouseenter':
                this.rowHoveredLast = action.rowIndex;
                break;
            case 'dblclick':
                this.rowHoveredLast = null;
                this.rowSelectedLast = null;
                break;
            default:
                console.warn('An unknown mouse event was passed to onRowMouseEvent');
        }
    };
    DataGridComponent.prototype.selectRow = function (row, rowIndex, isRightClick, elementRef) {
        if (this.options.selectionType) {
            var newRows = __spread(this.rowsInternal);
            if (this.keysPressed['control'] && this.options.selectionType === 'multi') {
                row.$$selected = row.$$selected ? false : true;
            }
            else if (this.keysPressed['shift'] && this.options.selectionType === 'multi' && this.rowSelectedLast) {
                newRows.forEach(function (rowNew) { return (rowNew.$$selected = false); });
                var startIndex = rowIndex > this.rowSelectedLast ? this.rowSelectedLast : rowIndex;
                var endIndex = rowIndex < this.rowSelectedLast ? this.rowSelectedLast : rowIndex;
                for (var i = startIndex; i < endIndex + 1; i++) {
                    if (newRows[i]) {
                        newRows[i].$$selected = true;
                    }
                }
            }
            else if (!row.$$selected && !isRightClick) {
                newRows.forEach(function (rowNew) { return (rowNew.$$selected = false); });
                row.$$selected = true;
            }
            else {
                if (isRightClick) {
                    if (!row.$$selected) {
                        newRows.forEach(function (rowNew) { return (rowNew.$$selected = false); });
                        row.$$selected = true;
                    }
                }
                else if (this.rowsSelectedCount > 1) {
                    newRows.forEach(function (rowNew) { return (rowNew.$$selected = false); });
                    row.$$selected = true;
                }
                else {
                    row.$$selected = false;
                }
            }
            var selectedRows = newRows.filter(function (rowNew) { return rowNew.$$selected; });
            this.rowSelectedLast = rowIndex;
            this.rowsSelectedCount = selectedRows.length;
            this.emitSelectedRows(selectedRows);
            this.onElementRef.emit(elementRef);
        }
    };
    DataGridComponent.prototype.getRowClasses = function (row) {
        if (!this.options.rowClass) {
            return null;
        }
        var classes = '';
        var results = this.options.rowClass(row);
        for (var key in results) {
            if (results.hasOwnProperty(key)) {
                if (results[key]) {
                    classes += key + ' ';
                }
            }
        }
        if (classes !== '') {
            return classes;
        }
    };
    DataGridComponent.prototype.createRowClasses = function () {
        var _this = this;
        var rowClasses = {};
        if (this.options.rowClass) {
            this.rows.forEach(function (row) {
                if (_this.options.primaryKey && row[_this.options.primaryKey] && _this.options.rowClass) {
                    var results = _this.options.rowClass(row);
                    var classes = '';
                    for (var key in results) {
                        if (results[key]) {
                            classes += key + ' ';
                        }
                    }
                    rowClasses[row[_this.options.primaryKey]] = classes.length ? classes : null;
                }
            });
        }
        this.rowClasses = rowClasses;
    };
    DataGridComponent.prototype.createRowStyles = function () {
        var _this = this;
        var rowStyles = {};
        if (!this.options.rowStyle) {
            this.rowStyles = rowStyles;
            return false;
        }
        var primaryKey = this.options.primaryKey ? this.options.primaryKey : '';
        if (!this.options.primaryKey) {
            console.log('Please specify a primary key to use rowStyles');
        }
        var stylesWithModels = [];
        var stylesNoModels = [];
        this.options.rowStyle.forEach(function (style) { return (style.model ? stylesWithModels.push(style.model) : stylesNoModels.push(style.rules)); });
        if (stylesNoModels.length && primaryKey) {
            this.rows.forEach(function (row) {
                stylesNoModels.forEach(function (rule) {
                    rowStyles[row[primaryKey]] = Object.assign(rowStyles[row[primaryKey]], rule(row));
                });
            });
        }
        if (stylesWithModels.length) {
            var subscription = combineLatest(stylesWithModels).subscribe(function (models) {
                rowStyles = {};
                if (stylesNoModels.length && primaryKey) {
                    _this.rows.forEach(function (row) {
                        stylesNoModels.forEach(function (rule) {
                            rowStyles[row[primaryKey]] = Object.assign(rowStyles[row[primaryKey]] || {}, rule(row));
                        });
                    });
                }
                _this.rows.forEach(function (row) {
                    models.forEach(function (model, index) {
                        if (_this.options &&
                            _this.options.rowStyle &&
                            _this.options.rowStyle[index] &&
                            _this.options.rowStyle[index].rules) {
                            if (_this.options.rowStyle && _this.options.rowStyle[index] && _this.options.rowStyle[index].rules) {
                                rowStyles[row[primaryKey]] = Object.assign({}, (rowStyles[row[primaryKey]] || {}), ((_this.options.rowStyle[index])).rules(row, model));
                            }
                        }
                    });
                });
                _this.rowStyles = rowStyles;
                _this.ref.detectChanges();
            });
            this.subscriptions.push((subscription));
        }
        this.rowStyles = rowStyles;
    };
    DataGridComponent.prototype.emitColumns = function () {
        var columnsNew = __spread(this.columnsPinnedLeft, this.columnsInternal);
        var columnsEmitted = columnsNew.map(function (column) {
            var columnNew = Object.assign({}, column);
            columnNew.locked = columnNew.pinnedLeft ? true : false;
            delete columnNew.templateCell;
            delete columnNew.templateHeader;
            return columnNew;
        });
        this.onColumnsUpdated.emit(columnsEmitted);
    };
    DataGridComponent.prototype.emitState = function (state) {
        var stateNew = Object.assign({}, state);
        delete ((stateNew)).usersColumns;
        var remapProps = JSON.parse(JSON.stringify(stateNew));
        remapProps.groups =
            remapProps.groups && remapProps.groups.length
                ? this.dgSvc.mapPropertiesUp(remapProps.groups, this.options.controlsMap)
                : [];
        remapProps.sorts =
            remapProps.sorts && remapProps.sorts.length
                ? this.dgSvc.mapPropertiesUp(remapProps.sorts, this.options.controlsMap)
                : [];
        remapProps.filters =
            remapProps.filters && remapProps.filters.length
                ? this.dgSvc.mapPropertiesUp(remapProps.filters, this.options.controlsMap)
                : [];
        this.onStateChange.emit(remapProps);
    };
    DataGridComponent.prototype.rowUpdated = function (event) {
        this.onRowUpdated.emit(event);
    };
    DataGridComponent.prototype.emitSelectedRows = function (rows) {
        this.onRowsSelected.emit(__spread(rows));
    };
    DataGridComponent.prototype.onUpdateDatatable = function (action) {
        if (action === 'update') {
            this.viewCreate();
        }
        else if (action === 'reset') {
            this.reset();
        }
    };
    DataGridComponent.prototype.customLinkEvent = function (data) {
        this.onCustomLinkEvent.emit(data);
    };
    DataGridComponent.prototype.reset = function (resetType) {
        this.ref.detach();
        if (resetType && resetType === 'groups') {
            this.state.groups = [];
        }
        else if (resetType && resetType === 'sorts') {
            this.state.sorts = [];
        }
        else if (resetType && resetType === 'filters') {
            this.state.filters = [];
        }
        else {
            this.state.groups = [];
            this.state.filters = [];
            this.state.sorts = [];
        }
        this.state.info = {};
        this.columnsInternal = this.columnsInternal.map(function (column) {
            column.pinnedLeft = false;
            column.locked = false;
            return Object.assign({}, column);
        });
        if (this.filterGlobal) {
            this.filterGlobal.term = '';
        }
        this.emitColumns();
        this.onStateUpdated({ action: Actions.reset, data: null });
        this.ref.reattach();
    };
    DataGridComponent.prototype.onWindowResize = function () {
        if (this.columnsInternal && this.columnsInternal.length && this.rowsInternal && this.rowsInternal.length) {
            this.viewCreate();
        }
    };
    DataGridComponent.prototype.ngOnDestroy = function () {
        this.subscriptions.forEach(function (sub) { return sub.unsubscribe(); });
    };
    return DataGridComponent;
}());
DataGridComponent.decorators = [
    { type: Component, args: [{
                selector: 'datagrid',
                template: "\n<div id=\"datagrid\" class=\"datagrid\" #dataGrid\n\t [ngClass]=\"{\n    'selectable' : options.selectionType,\n    'truncatable' : options.columnsTruncate,\n    'fullscreen' : options.fullScreen }\">\n\n\t<div *ngIf=\"!appReady || !domReady\" class=\"container-fluid pt-3\" style=\"font-size: 1rem;\"><i class=\"fa fa-spinner fa-spin\"></i> Loading View...</div>\n\n\t<div class=\"datagrid-container\" *ngIf=\"appReady && domReady\">\n\n\t\t<!--TODO: Add readonly div -->\n\t\t<!-- Header -->\n\t\t<datagrid-header [columns]=\"columnsExternal\"\n\t\t\t\t\t\t [columnsPinnedLeft]=\"columnsPinnedLeft\"\n\t\t\t\t\t\t [state]=\"state\"\n\t\t\t\t\t\t [status]=\"status\"\n\t\t\t\t\t\t [options]=\"options\"\n\t\t\t\t\t\t [scrollProps]=\"scrollProps\"\n\t\t\t\t\t\t [gridProps]=\"gridProps\"\n\t\t\t\t\t\t [filterTerms]=\"filterTerms\"\n             [templates]=\"columnTemplates\"\n\t\t\t\t\t\t (onStateUpdated)=\"onStateUpdated($event)\"\n\t\t\t\t\t\t (onColumnsUpdated)=\"columnsUpdated($event)\"\n\t\t\t\t\t\t (onCustomLinkEvent)=\"customLinkEvent($event)\">\n\t\t</datagrid-header>\n\n\t\t<!-- Info bar-->\n\t\t<datagrid-info *ngIf=\"options.showInfo && (state?.sorts.length || state?.groups.length || state?.filters.length)\"\n\t\t\t\t\t   [state]=\"state\"\n\t\t\t\t\t   [options]=\"options\"\n\t\t\t\t\t   [columnsMapped]=\"columnsMapped\"\n\t\t\t\t\t   (onReset)=\"reset($event)\">\n\t\t</datagrid-info>\n\n\t\t<!-- Body -->\n\t\t<datagrid-body #dataGridBody\n\t\t\t\t\t   [columns]=\"columnsExternal\"\n\t\t\t\t\t   [columnsPinnedLeft]=\"columnsPinnedLeft\"\n\t\t\t\t\t   [rows]=\"rowsExternal\"\n\t\t\t\t\t   [rowStyles]=\"rowStyles\"\n\t\t\t\t\t   [state]=\"state\"\n\t\t\t\t\t   [status]=\"status\"\n\t\t\t\t\t   [options]=\"options\"\n\t\t\t\t\t   [gridProps]=\"gridProps\"\n             [templates]=\"columnTemplates\"\n\t\t\t\t\t   (onRowMouseEvent)=\"onRowMouseEvent($event)\"\n\t\t\t\t\t   (onRowUpdated)=\"rowUpdated($event)\"\n\t\t\t\t\t   (onGroupToggled)=\"groupToggled($event)\"\n\t\t\t\t\t   (onScrollEvent)=\"onScroll($event)\">\n\t\t</datagrid-body>\n\t</div>\n\n\t<!-- Drag select box -->\n\t<div id=\"drag-select\"\n\t\t *ngIf=\"options.showDragBox && dragging && options.selectionType && draggingPos.hasMinSize\"\n\t\t [ngStyle]=\"{ 'top':draggingPos.top + 'px', 'left': draggingPos.left + 'px', width: draggingPos.width + 'px', height: draggingPos.height + 'px' }\">\n\t</div>\n\n</div>\n",
                styles: ["#datagrid{font-size:12px;border-top:1px solid #ccc;position:relative}#datagrid.maxHeight{overflow-y:scroll;overflow-x:hidden}#datagrid.fullscreen{position:absolute;top:0;bottom:0;left:0;right:0}#datagrid.scrollH{overflow-x:scroll}#datagrid.selectable{-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}#datagrid.selectable .datagrid-body-row:hover:before{content:'';display:block;position:absolute;top:-1px;height:2px;z-index:999;left:0;right:0;background:#000}#datagrid.selectable .datagrid-body-row:hover:after{content:'';display:block;position:absolute;bottom:-2px;height:2px;z-index:999;left:0;right:0;background:#000}#datagrid.selectable .datagrid-body-row.selected,#datagrid.selectable .datagrid-body-row.selected .datagrid-body-pinned{background-color:#ececec}#datagrid.selectable .datagrid-body-row.selected:before{content:'';display:block;position:absolute;top:-1px;height:2px;z-index:999;left:0;right:0;background:red}#datagrid.selectable .datagrid-body-row.selected:after{content:'';display:block;position:absolute;bottom:-2px;height:2px;z-index:999;left:0;right:0;background:red}#datagrid.selectable .datagrid-body-row.selected+.datagrid-body-row.selected:before{display:none!important}#datagrid .sticky-left{position:-webkit-sticky;position:sticky;left:0}#datagrid #drag-select{width:20px;height:20px;background:red;position:fixed;opacity:.3;z-index:1000}#datagrid .datagrid-container{margin:0;background-color:#fff;overflow:hidden;min-height:200px}#datagrid .datagrid-container .datagrid-header-container{height:24px;position:relative;z-index:4;background-image:-webkit-gradient(linear,left bottom,left top,from(#dadada),to(#f1f1f1));background-image:linear-gradient(to top,#dadada 0,#f1f1f1 100%)}#datagrid .datagrid-container .datagrid-header .datagrid-header-row{background-color:#f1f1f1;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-flow:row;flex-flow:row;-webkit-backface-visibility:hidden;backface-visibility:hidden}#datagrid .datagrid-container .datagrid-header .datagrid-header-pinned{left:0;z-index:7!important;background-color:#f1f1f1;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-flow:row;flex-flow:row}#datagrid .datagrid-container .datagrid-header .datagrid-header-main{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-flow:row;flex-flow:row;position:absolute}#datagrid .datagrid-container .datagrid-header .datagrid-header-cell{line-height:20px;font-weight:700;display:inline-block;cursor:default;background-color:#f1f1f1;background-image:-webkit-gradient(linear,left bottom,left top,from(#dadada),to(#f1f1f1));background-image:linear-gradient(to top,#dadada 0,#f1f1f1 100%);white-space:nowrap;padding:0;outline:#ccc solid 1px;top:0;position:absolute;z-index:6;border:0}#datagrid .datagrid-container .datagrid-header .datagrid-header-cell.pinned{z-index:7!important;background-color:#f1f1f1}#datagrid .datagrid-container .datagrid-header .datagrid-header-cell.reorderable .datagrid-header-cell-label{cursor:grab;cursor:-webkit-grab}#datagrid .datagrid-container .datagrid-header .datagrid-header-cell.column-last .dropdown-submenu-flyout{left:auto!important;right:100%!important}#datagrid .datagrid-container .datagrid-header .datagrid-header-cell>div{padding-right:8px}#datagrid .datagrid-container .datagrid-header .datagrid-header-cell .resize-handle{display:block;position:absolute;background:#f6ac38;top:0;bottom:0;right:-2px;width:5px;z-index:8;cursor:ew-resize}#datagrid .datagrid-container .datagrid-header .datagrid-header-cell .datagrid-header-cell-label{padding:2px 8px;overflow:hidden;display:block}#datagrid .datagrid-container .datagrid-header .datagrid-header-cell a.active{color:red!important}#datagrid .datagrid-container .datagrid-header .datagrid-header-cell .header-controls{display:inline-block!important;position:absolute;right:0;padding-right:5px;padding-left:5px;background-color:#f1f1f1;background-image:-webkit-gradient(linear,left bottom,left top,from(#dadada),to(#f1f1f1));background-image:linear-gradient(to top,#dadada 0,#f1f1f1 100%);top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%)}#datagrid .datagrid-container .datagrid-header .dropdown .active{color:red;background:0 0}#datagrid .datagrid-container .datagrid-header .dropdown .dropdown-item{padding:3px .75rem;font-size:.8rem}#datagrid .datagrid-container .datagrid-header .dropdown .dropdown-item:hover{background-color:#f7f7f9}#datagrid .datagrid-container .datagrid-header .dropdown .disabled:hover{background-color:transparent}#datagrid .datagrid-container .datagrid-header .dropdown>.dropdown-menu{top:20px!important;min-width:200px;font-size:.8rem;padding:.5rem 0}#datagrid .datagrid-container .datagrid-header .dropdown .dropdown-submenu{position:relative;display:block;font-size:.8rem}#datagrid .datagrid-container .datagrid-header .dropdown .dropdown-submenu .dropdown-submenu-button:after{font-family:FontAwesome;content:'\\f0da';position:absolute;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);right:10px}#datagrid .datagrid-container .datagrid-header .dropdown .dropdown-submenu .dropdown-submenu-flyout{left:100%;top:-25px!important;display:none;padding-top:0}#datagrid .datagrid-container .datagrid-header .dropdown .dropdown-submenu .dropdown-submenu-flyout .btn-group{width:100%}#datagrid .datagrid-container .datagrid-header .dropdown .dropdown-submenu .dropdown-submenu-flyout .btn{width:50%}#datagrid .datagrid-container .datagrid-header .dropdown .dropdown-submenu.dropdown-submenu-left .dropdown-submenu-flyout{right:100%;left:auto}#datagrid .datagrid-container .datagrid-header .dropdown .dropdown-submenu:hover .dropdown-submenu-flyout{display:block;padding:.5rem 0}#datagrid .datagrid-container .datagrid-header .dropdown .dropdown-submenu .terms-list{max-height:300px;overflow-y:auto;padding-bottom:.75rem}#datagrid .datagrid-container .datagrid-header .dropdown .dropdown-submenu .terms-list .term{font-weight:400;font-size:.9rem;padding:0 2rem}#datagrid .datagrid-container .datagrid-header .dropdown .dropdown-submenu .terms-list label.term:hover{background-color:#f7f7f9}#datagrid .datagrid-container .datagrid-header .dropdown .dropdown-submenu .box{padding:0 1rem}#datagrid .datagrid-container .datagrid-body-container{overflow:scroll;position:relative;z-index:3}#datagrid .datagrid-container .datagrid-body-container.widthFixed{overflow-x:hidden}#datagrid .datagrid-container .datagrid-info-row{line-height:23px;padding-left:.5rem;border-bottom:1px solid #ccc;background-color:#ededed;-webkit-box-shadow:5px 1px 5px rgba(0,0,0,.3);box-shadow:5px 1px 5px rgba(0,0,0,.3);overflow:initial}#datagrid .datagrid-container .datagrid-body .datagrid-body-row{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-flow:row;flex-flow:row;-webkit-backface-visibility:hidden;backface-visibility:hidden;position:absolute;border-bottom:1px solid #b3b3b3;height:24px}#datagrid .datagrid-container .datagrid-body .datagrid-body-pinned{left:0;background-color:#fff;z-index:4;display:inline-block;border-right:1px solid #ccc}#datagrid .datagrid-container .datagrid-body .datagrid-body-main{min-height:23.5px}#datagrid .datagrid-container .datagrid-body .datagrid-row{position:relative;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-flow:row;flex-flow:row;min-height:23px}#datagrid .datagrid-container .datagrid-body .datagrid-row.selected-state .datagrid-cell{background:#f0dada}#datagrid .datagrid-container .datagrid-body .datagrid-row .datagrid-cell{position:absolute;min-height:23px;border:none}#datagrid .datagrid-container .datagrid-body .datagrid-row .datagrid-cell-data{line-height:23px;position:relative;z-index:3;display:inline-block;white-space:nowrap;text-overflow:ellipsis;min-height:23px;overflow:hidden}#datagrid .datagrid-container .datagrid-body .datagrid-row .datagrid-cell-data .datagrid-cell-data.canEdit:hover{outline:#e0e0e0 dashed 2px;outline-offset:-2px}#datagrid .datagrid-container .datagrid-body .datagrid-row .datagrid-cell-data .datagrid-cell-data-content{position:absolute;padding:0 5px;left:0;right:0;bottom:0;top:0}#datagrid .datagrid-container .datagrid-body .datagrid-row .datagrid-cell-data.pinned{-webkit-box-shadow:2px 5px 5px rgba(0,0,0,.3);box-shadow:2px 5px 5px rgba(0,0,0,.3);background-color:#fff;z-index:4}#datagrid .datagrid-container .datagrid-body .datagrid-row .datagrid-cell-data .cell{height:22px;padding:3px 8px}#datagrid .datagrid-container .datagrid-body .datagrid-row .datagrid-cell-data .cell-tooltip{position:absolute;left:0;right:0;top:0;bottom:0}#datagrid .datagrid-container .datagrid-body .datagrid-truncated{width:5px;height:5px;background:red;-webkit-transform:rotate(45deg);transform:rotate(45deg);position:absolute;top:-2px;right:-2px;overflow:hidden}#datagrid .datagrid-container .datagrid-body .datagrid-group-header{min-height:24px}#datagrid .datagrid-container .datagrid-body .datagrid-group-header .datagrid-group-header-cell{padding:0;z-index:4;left:0}#datagrid .datagrid-container .datagrid-body .datagrid-group-header a{background-image:-webkit-gradient(linear,left bottom,left top,from(#fcfeff),to(#cbdce6));background-image:linear-gradient(to top,#fcfeff 0,#cbdce6 100%);padding:.15rem .15rem .15rem .5rem;display:block;border-bottom:1px solid #c2d1da}#datagrid .datagrid-container .datagrid-body .datagrid-group-header a>span{left:16px}#datagrid .datagrid-container .datagrid-body .datagrid-group-header a>span:before{font-family:FontAwesome;content:'\\f0da';-webkit-transition:.1s ease-out;transition:.1s ease-out;display:inline-block;margin-right:5px}#datagrid .datagrid-container .datagrid-body .datagrid-group-header a.active span:before{-webkit-transform:rotate(90deg);transform:rotate(90deg);-webkit-transition:.1s ease-out;transition:.1s ease-out}#datagrid .fa-stack-groupsort{margin-right:-5px}#datagrid .fa-stack-groupsort .arrow{margin-right:-3px;font-size:104%}#datagrid .fa-stack-groupsort .group{position:relative;top:-1px}#datagrid .anim-fade-in{opacity:0;-webkit-animation:.3s ease-in forwards fade;animation:.3s ease-in forwards fade;position:relative}@-webkit-keyframes fade{from{opacity:0}to{opacity:1}}@keyframes fade{from{opacity:0}to{opacity:1}}"],
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    '(document:keydown)': 'handleKeyboardEvents($event)',
                    '(document:keyup)': 'handleKeyboardEvents($event)',
                    '(window:resize)': 'onWindowResizeThrottled($event)',
                },
            },] },
];
DataGridComponent.ctorParameters = function () { return [
    { type: DataGridService, },
    { type: ChangeDetectorRef, },
]; };
DataGridComponent.propDecorators = {
    "dataGrid": [{ type: ViewChild, args: ['dataGrid',] },],
    "dataGridBody": [{ type: ViewChild, args: ['dataGridBody',] },],
    "columns": [{ type: Input },],
    "rows": [{ type: Input },],
    "state": [{ type: Input },],
    "columnTemplates": [{ type: ContentChildren, args: [DataTableColumnDirective,] },],
    "options": [{ type: Input },],
    "filterGlobal": [{ type: Input },],
    "onColumnsUpdated": [{ type: Output },],
    "onRowsSelected": [{ type: Output },],
    "onStateChange": [{ type: Output },],
    "onRightClickMenu": [{ type: Output },],
    "action": [{ type: Output },],
    "onCustomLinkEvent": [{ type: Output },],
    "onElementRef": [{ type: Output },],
    "onRowUpdated": [{ type: Output },],
};
var HeaderComponent = /** @class */ (function () {
    function HeaderComponent() {
        this.onColumnsUpdated = new EventEmitter();
        this.onStateUpdated = new EventEmitter();
        this.onCustomLinkEvent = new EventEmitter();
    }
    HeaderComponent.prototype.ngOnInit = function () { };
    HeaderComponent.prototype.stateUpdated = function (event) {
        this.onStateUpdated.emit(event);
    };
    HeaderComponent.prototype.columnsUpdated = function (event) {
        this.onColumnsUpdated.emit(event);
    };
    HeaderComponent.prototype.customLinkEvent = function (data) {
        this.onCustomLinkEvent.emit(data);
    };
    return HeaderComponent;
}());
HeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'datagrid-header',
                template: "<div class=\"datagrid-header-container\" [ngStyle]=\"{'width': gridProps.widthViewPort + 'px'}\">\n    <div class=\"datagrid-header\" [ngStyle]=\"{'width' : gridProps.widthTotal + 'px'}\">\n\n        <!-- Header Row -->\n        <div class=\"datagrid-header-row\">\n\n            <div class=\"datagrid-header-pinned sticky-left\" *ngIf=\"columnsPinnedLeft?.length\" [ngStyle]=\"{'width' : gridProps.widthPinned + 'px' }\">\n                <datagrid-header-row [columns]=\"columnsPinnedLeft\"\n                                     [columnType]=\"'pinnedLeft'\"\n                                     [state]=\"state\"\n                                     [status]=\"status\"\n                                     [options]=\"options\"\n                                     [scrollProps]=\"scrollProps\"\n                                     [filterTerms]=\"filterTerms\"\n                                     [templates]=\"templates\"\n                                     (onColumnsUpdated)=\"columnsUpdated($event)\"\n                                     (onStateUpdated)=\"stateUpdated($event)\"\n                                     (onCustomLinkEvent)=\"customLinkEvent($event)\">\n                </datagrid-header-row>\n            </div>\n            \n            <div class=\"datagrid-header-main\" [ngStyle]=\"{ 'left' : (-scrollProps.scrollLeft + gridProps.widthPinned) + 'px' }\">\n                <datagrid-header-row [columns]=\"columns\"\n                                     [columnType]=\"'main'\"\n                                     [state]=\"state\"\n                                     [status]=\"status\"\n                                     [options]=\"options\"\n                                     [scrollProps]=\"scrollProps\"\n                                     [filterTerms]=\"filterTerms\"\n                                     [templates]=\"templates\"\n                                     (onColumnsUpdated)=\"columnsUpdated($event)\"\n                                     (onStateUpdated)=\"stateUpdated($event)\"\n                                     (onCustomLinkEvent)=\"customLinkEvent($event)\">\n                </datagrid-header-row>\n            </div>\n\n        </div>\n    </div>\n</div>\n",
                styles: [],
                changeDetection: ChangeDetectionStrategy.OnPush,
            },] },
];
HeaderComponent.ctorParameters = function () { return []; };
HeaderComponent.propDecorators = {
    "columns": [{ type: Input },],
    "columnsPinnedLeft": [{ type: Input },],
    "state": [{ type: Input },],
    "status": [{ type: Input },],
    "options": [{ type: Input },],
    "gridProps": [{ type: Input },],
    "scrollProps": [{ type: Input },],
    "filterTerms": [{ type: Input },],
    "templates": [{ type: Input },],
    "onColumnsUpdated": [{ type: Output },],
    "onStateUpdated": [{ type: Output },],
    "onCustomLinkEvent": [{ type: Output },],
};
var FiltersComponent = /** @class */ (function () {
    function FiltersComponent() {
        this.filterTerms = {};
        this.onFiltersUpdated = new EventEmitter();
        this.filterTerm = new BehaviorSubject(null);
        this.subs = [];
    }
    FiltersComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.filterTerm.pipe(debounceTime(300)).subscribe(function (filter) {
            if (filter) {
                _this.onFiltersUpdated.emit(filter);
            }
        });
        if (this.options.columnData) {
            var sub = this.options.columnData[this.column.prop].model.subscribe(function (model) {
                if (_this.options.columnData) {
                    _this.model = _this.options.columnData[_this.column.prop].modelSrc
                        ? model[_this.options.columnData[_this.column.prop].modelSrc]
                        : model;
                    _this.modelLabel = _this.options.columnData[_this.column.prop].label || '';
                    _this.modelValue = _this.options.columnData[_this.column.prop].value || '';
                    _this.modelClasses = _this.options.columnData[_this.column.prop].classes || '';
                    _this.modelStyles = _this.options.columnData[_this.column.prop].styles || '';
                }
            });
            this.subs.push(sub);
        }
    };
    FiltersComponent.prototype.modifyFilter = function (columnProp, operator, newTerm, oldTerm) {
        var filterObj = {
            filterAction: '',
            filter: { prop: columnProp, operator: operator, value: newTerm },
        };
        if ((oldTerm === '' || !oldTerm) && newTerm !== '') {
            filterObj.filterAction = 'add';
        }
        else if (newTerm === '' || newTerm === false || !newTerm || newTerm === oldTerm) {
            filterObj.filterAction = 'remove';
        }
        else if (newTerm !== oldTerm) {
            filterObj.filterAction = 'change';
        }
        this.filterTerm.next(filterObj);
    };
    FiltersComponent.prototype.ngOnDestroy = function () {
        if (this.subs.length) {
            this.subs.forEach(function (sub) { return sub.unsubscribe(); });
        }
    };
    return FiltersComponent;
}());
FiltersComponent.decorators = [
    { type: Component, args: [{
                selector: '<filters>',
                template: "<!--\n<div class=\"btn-group\" role=\"group\">\n        <button type=\"button\" class=\"btn btn-interact-outline\"><i aria-hidden=\"true\" class=\"fa fa-filter\"></i> Filter</button>\n        <button type=\"button\" class=\"btn btn-interact-outline\"><i aria-hidden=\"true\" class=\"fa fa-remove\"></i> Clear</button>\n</div>\n-->\n\n <!-- Filtering -->\n <div *ngIf=\"column.canSearch\">\n    <label class=\"icon filter ml-2 mr-2 mt-2\" *ngIf=\"status.filters[column.prop]\">\n        <input class=\"form-control form-control-sm\" placeholder=\"Filter\" name=\"filterSearch\" [ngModel]=\"status.filters[column.prop].contains\" (ngModelChange)=\"modifyFilter(column.prop, 'contains', $event, status.filters[column.prop].contains)\"  [attr.data-qa]=\"'controls-filter-search'\"/>\n        <a class=\"clear\" (click)=\"modifyFilter(column.prop, 'contains', '', status.filters[column.prop].contains)\" *ngIf=\"status.filters[column.prop].contains != ''\">\n            <i class=\"fa fa-remove\"></i></a>\n    </label>\n</div>\n\n<!-- Strings -->\n<ng-container *ngIf=\"column.columnType == 'string'\">\n    <div class=\"terms-list\" [attr.data-qa]=\"'controls-filter-terms'\">\n        <ng-container *ngIf=\"options?.columnData && options?.columnData[column.prop];else filterList\">\n            <label class=\"term form-check-label field-{{column.prop + '-' + term[modelValue]}} {{term[modelClasses]}}\" [ngStyle]=\"term[modelStyles]\" *ngFor=\"let term of model\">\n                <input type=\"checkbox\" class=\"form-check-input\" [value]=\"term[modelValue]\"\n                [ngModel]=\"status.filters[column.prop]?.eq[term[modelValue]]\" (ngModelChange)=\"modifyFilter(column.prop, 'eq', term[modelValue], status.filters[column.prop].eq[term[modelValue]])\">\n                <span>{{term[modelLabel]}}</span>\n            </label>\n        </ng-container>\n        <ng-template #filterList>\n            <ng-container *ngIf=\"filterTerms[this.column.prop]?.length\">\n                <label class=\"term form-check-label\" *ngFor=\"let term of filterTerms[this.column.prop]\">\n                    <input type=\"checkbox\" class=\"form-check-input\" [value]=\"term\" [attr.data-qa]=\"'controls-filter-term' + term\"\n                           [ngModel]=\"status.filters[column.prop]?.eq[term]\" (ngModelChange)=\"modifyFilter(column.prop, 'eq', term, status.filters[column.prop].eq[term])\">\n                    <span>{{term}}</span>\n                </label>\n            </ng-container>\n            <span *ngIf=\"!filterTerms[this.column.prop]?.length\" class=\"term\">No filter terms found</span>\n        </ng-template>\n        \n    </div>\n</ng-container>\n\n<!-- Numbers and Money \n<ng-container *ngIf=\"column.columnType == 'int' || column.columnType == 'money'\">\n\t<h6 class=\"dropdown-header\">Show items with values that:</h6>\n\t<div class=\"box\">\n\t\t<div class=\"form-group\">\n\t\t\t<select class=\"form-control form-control-sm mb-1\">\n\t\t\t\t<option>Is Equal To</option>\n\t\t\t\t<option>Is Not Equal To</option>\n\t\t\t\t<option>Is Greater Than Or Equal To</option>\n\t\t\t\t<option>Is Greater Than</option>\n\t\t\t\t<option>Is Less Than Or Equal To</option>\n\t\t\t\t<option>Is Less Than</option>\n\t\t\t\t<option>Is Null</option>\n\t\t\t\t<option>Is Not Null</option>\n\t\t\t</select>\n\t\t\t<input class=\"form-control form-control-sm\" type=\"number\" />\n\t\t</div>\n\t\t<div class=\"form-group\">\n\t\t\t<select class=\"form-control form-control-sm\">\n\t\t\t\t<option>And</option>\n\t\t\t\t<option>Or</option>\n\t\t\t</select>\n\t\t</div>\n\t\t<div class=\"form-group\">\n\t\t\t<select class=\"form-control form-control-sm mb-1\">\n\t\t\t\t<option>Is Equal To</option>\n\t\t\t\t<option>Is Not Equal To</option>\n\t\t\t\t<option>Is Greater Than Or Equal To</option>\n\t\t\t\t<option>Is Greater Than</option>\n\t\t\t\t<option>Is Less Than Or Equal To</option>\n\t\t\t\t<option>Is Less Than</option>\n\t\t\t\t<option>Is Null</option>\n\t\t\t\t<option>Is Not Null</option>\n\t\t\t</select>\n\t\t\t<input class=\"form-control form-control-sm\" type=\"number\" />\n\t\t</div>\n\t</div>\n</ng-container>\n-->\n\n<!-- Dates\n<ng-container *ngIf=\"column.columnType == 'date'\">\n    <h6 class=\"dropdown-header\">Show items with values that:</h6>\n    <div class=\"box\">\n        <div class=\"form-group\">\n            <select class=\"form-control form-control-sm mb-1\" [(ngModel)]=\"model\" >\n                <option>Is Equal To</option>\n                <option>Is Not Equal To</option>\n                <option>Is Greater Than Or Equal To</option>\n                <option>Is Greater Than</option>\n                <option>Is Less Than Or Equal To</option>\n                <option>Is Less Than</option>\n                <option>Is Null</option>\n                <option>Is Not Null</option>\n            </select>\n            <form class=\"form-inline\">\n                <div class=\"form-group w-100\">\n                    <div class=\"input-group\">\n                        <input class=\"form-control\" placeholder=\"yyyy-mm-dd\"\n                               name=\"dp\" [(ngModel)]=\"model\" ngbDatepicker #d=\"ngbDatepicker\">\n                        <button class=\"input-group-addon\" (click)=\"d.toggle()\" type=\"button\">\n                            <i class=\"fa fa-calendar\"></i>\n                        </button>\n                    </div>\n                </div>\n            </form>\n        </div>\n        <div class=\"form-group\">\n            <select class=\"form-control form-control-sm\">\n                <option>And</option>\n                <option>Or</option>\n            </select>\n        </div>\n        <div class=\"form-group\">\n            <select class=\"form-control form-control-sm mb-1\">\n                <option>Is Equal To</option>\n                <option>Is Not Equal To</option>\n                <option>Is Greater Than Or Equal To</option>\n                <option>Is Greater Than</option>\n                <option>Is Less Than Or Equal To</option>\n                <option>Is Less Than</option>\n                <option>Is Null</option>\n                <option>Is Not Null</option>\n            </select>\n            <form class=\"form-inline\">\n                <div class=\"form-group w-100\">\n                    <div class=\"input-group\">\n                        <input class=\"form-control\" placeholder=\"yyyy-mm-dd\"\n                               name=\"dp\" [(ngModel)]=\"model2\" ngbDatepicker #d2=\"ngbDatepicker\">\n                        <button class=\"input-group-addon\" (click)=\"d.toggle()\" type=\"button\">\n                            <i class=\"fa fa-calendar\"></i>\n                        </button>\n                    </div>\n                </div>\n            </form>\n        </div>\n        \n    </div>\n</ng-container>\n-->\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
            },] },
];
FiltersComponent.ctorParameters = function () { return []; };
FiltersComponent.propDecorators = {
    "column": [{ type: Input },],
    "options": [{ type: Input },],
    "state": [{ type: Input },],
    "status": [{ type: Input },],
    "filterTerms": [{ type: Input },],
    "onFiltersUpdated": [{ type: Output },],
};
var ControlsComponent = /** @class */ (function () {
    function ControlsComponent() {
        this.onStateUpdated = new EventEmitter();
        this.onCustomLinkEvent = new EventEmitter();
        this.subs = [];
    }
    ControlsComponent.prototype.ngOnInit = function () { };
    ControlsComponent.prototype.modifySorts = function (action, prop, direction) {
        if (!direction) {
            direction = 'asc';
        }
        else if (direction === 'asc') {
            direction = 'desc';
        }
        else if (direction === 'desc') {
            direction = null;
        }
        this.modifyState(action, { dir: direction, prop: prop });
    };
    ControlsComponent.prototype.modifyPinned = function (action, column, index) {
        this.modifyState(action, { prop: column.prop, index: index, isPinned: column.pinnedLeft });
    };
    ControlsComponent.prototype.modifyFilters = function (data) {
        this.modifyState(Actions.filter, data);
    };
    ControlsComponent.prototype.clearFilters = function (columnProp) {
        this.modifyState(Actions.filter, { filterAction: 'clear', filter: { prop: columnProp } });
    };
    ControlsComponent.prototype.modifyState = function (action, data) {
        this.onStateUpdated.emit({ action: action, data: data });
    };
    ControlsComponent.prototype.customLinkAction = function (link, column) {
        this.onCustomLinkEvent.emit({ link: link, column: column });
    };
    ControlsComponent.prototype.deleteColumn = function (columnIndex) {
        this.modifyState(Actions.column, { action: 'delete', columnIndex: columnIndex });
    };
    ControlsComponent.prototype.ngOnDestroy = function () {
        if (this.subs.length) {
            this.subs.forEach(function (sub) { return sub.unsubscribe(); });
        }
    };
    return ControlsComponent;
}());
ControlsComponent.decorators = [
    { type: Component, args: [{
                selector: 'datagrid-controls',
                template: "<div ngbDropdown id=\"headerComponentDropdown\" class=\"header-controls d-inline-block dropdown\" #myDrop=\"ngbDropdown\" placement=\"{{columnIndex < 1 ? 'bottom-left' : 'bottom-right'}}\">\n    <a ngbDropdownToggle *ngIf=\"status?.groups || status?.sorts || status?.filters\" [attr.data-qa]=\"'controls-dropdown'\"\n       [ngClass]=\"{'active' : status.groups[column.prop] || status.sorts[column.prop] || status.filters[column.prop]?.hasFilters || column.pinnedLeft }\">\n        <i class=\"fa fa-bars\" aria-hidden=\"true\"></i>\n    </a>\n    <div ngbDropdownMenu aria-labelledby=\"dropdownBasic1\">\n\n        <!-- Sorting -->\n        <button class=\"dropdown-item\" *ngIf=\"column.canSort && status?.sorts\" (click)=\"modifySorts('sort', column.prop, status.sorts[column.prop])\" [attr.data-qa]=\"'controls-sort'\"\n                [ngClass]=\"{'active' : status.sorts[column.prop] }\">\n            <ng-container *ngIf=\"status.sorts[column.prop] == 'asc'\">\n                <span class=\"fa-stack fa-stack-groupsort\">\n                    <i class=\"fa fa-fw fa-sort-amount-asc\" aria-hidden=\"true\"></i>\n                </span> Sorted Ascending\n            </ng-container>\n            <ng-container *ngIf=\"status.sorts[column.prop] == 'desc'\">\n                <span class=\"fa-stack fa-stack-groupsort\">\n                    <i class=\"fa fa-fw fa-sort-amount-asc fa-flip-vertical\" aria-hidden=\"true\"></i>\n                </span> Sorted Descending\n            </ng-container>\n            <ng-container *ngIf=\"!status.sorts[column.prop]\">\n                <span class=\"fa-stack fa-stack-groupsort\">\n                    <i class=\"fa fa-fw fa-sort-amount-desc\" aria-hidden=\"true\"></i>\n                </span> Sort By {{column.label}}\n            </ng-container>\n        </button>\n\n        <!-- Grouping -->\n        <button class=\"dropdown-item\" *ngIf=\"column.canGroup && status?.groups\" (click)=\"modifySorts('group', column.prop, status.groups[column.prop])\" [attr.data-qa]=\"'controls-group'\"\n                [ngClass]=\"{'active' : status.groups[column.prop] }\">\n            <ng-container *ngIf=\"status.groups[column.prop] == 'asc'\">\n                <span class=\"fa-stack fa-stack-groupsort\">\n                    <i class=\"fa arrow fa-long-arrow-down\" aria-hidden=\"true\"></i>\n                    <i class=\"fa fa-object-group\" aria-hidden=\"true\"></i>\n                </span> Grouped Ascending\n            </ng-container>\n            <ng-container *ngIf=\"status.groups[column.prop] == 'desc'\">\n                <span class=\"fa-stack fa-stack-groupsort\">\n                    <i class=\"fa arrow fa-long-arrow-up\" aria-hidden=\"true\"></i>\n                    <i class=\"fa fa-object-group\" aria-hidden=\"true\"></i>\n                </span> Grouped Descending\n            </ng-container>\n            <ng-container *ngIf=\"!status.groups[column.prop]\">\n                <span class=\"fa-stack fa-stack-groupsort\">\n                    <i class=\"fa arrow fa-long-arrow-down\" aria-hidden=\"true\"></i>\n                    <i class=\"fa fa-object-group\" aria-hidden=\"true\"></i>\n                </span> Group By {{column.label}}\n            </ng-container>\n        </button>\n\n        <!-- Pinning -->\n        <button class=\"dropdown-item\" *ngIf=\"column.canPinLeft\" (click)=\"myDrop.close();modifyPinned('pinLeft', column, columnIndex)\" [attr.data-qa]=\"'controls-pinLeft'\"\n                [ngClass]=\"{'active' : column.pinnedLeft }\">\n            <ng-container *ngIf=\"column.pinnedLeft\">\n                <span class=\"fa-stack fa-stack-groupsort\">\n                    <i class=\"fa fa-caret-square-o-left\" aria-hidden=\"true\"></i>\n                </span> Pinned\n            </ng-container>\n            <ng-container *ngIf=\"!column.pinnedLeft\">\n                <span class=\"fa-stack fa-stack-groupsort\">\n                    <i class=\"fa fa-caret-square-o-left\" aria-hidden=\"true\"></i>\n                </span> Pin {{column.label}}\n            </ng-container>\n        </button>\n\n        <!-- Filtering -->\n        <div class=\"dropdown-submenu\" *ngIf=\"(column.canFilter || column.canSearch) && status?.filters\" [attr.data-qa]=\"'controls-filter'\"\n             [ngClass]=\"{'dropdown-submenu-left': columnIndex > (columnsCount - 5)}\" (click)=\"$event.stopPropagation()\">\n            <button class=\"dropdown-item dropdown-submenu-button\" [attr.data-qa]=\"'controls-filter-dropdown'\"\n                    [ngClass]=\"{'active' :  status.filters[column.prop]?.hasFilters }\">\n                <span class=\"fa-stack fa-stack-groupsort\">\n                    <i class=\"fa fa-filter\" aria-hidden=\"true\"></i>\n                </span> Filters\n            </button>\n            <div class=\"dropdown-menu dropdown-submenu-flyout\">\n\n                <filters\n                    [column]=\"column\"\n                    [state]=\"state\"\n                    [status]=\"status\"\n                    [options]=\"options\"\n                    [filterTerms]=\"filterTerms\"\n                    (onFiltersUpdated)=\"modifyFilters($event)\">\n                </filters>\n                \n                <div class=\"pl-2 pt-1 pr-2 pb-0\">\n                    <button class=\"btn btn-interact-outline btn-sm mt-2 w-100\" (click)=\"clearFilters(column.prop)\" [attr.data-qa]=\"'controls-filter-clear'\">Clear Filters</button>\n                </div>\n            </div>\n        </div>\n\n        <!-- Custom Links -->\n        <ng-container *ngIf=\"column?.controlsCustomLinks?.length\">\n            <div class=\"dropdown-divider\"></div>\n            <!-- Loop through custom links -->\n                <ng-container *ngFor=\"let link of column.controlsCustomLinks\">\n                    <div class=\"dropdown-submenu\"\n                         [ngClass]=\"{'dropdown-submenu-left': columnIndex > (columnsCount - 5) }\"\n                         [attr.data-qa]=\"'controls-custom-link-dropdown'\">\n                        <button class=\"dropdown-item\" [attr.data-qa]=\"'controls-custom-link'\" [ngClass]=\"{ 'dropdown-submenu-button' : link?.submenu?.length }\">\n                            <span class=\"fa-stack fa-stack-groupsort\">\n                                <i class=\"{{link.icon}}\" aria-hidden=\"true\" *ngIf=\"link.icon\"></i>\n                            </span> {{link.label}}\n                        </button>\n                        <!-- If submenu links, show flyout -->\n                        <div class=\"dropdown-menu dropdown-submenu-flyout\" *ngIf=\"link?.submenu?.length\">\n                            <button class=\"dropdown-item\" [attr.data-qa]=\"'controls-custom-link'\" *ngFor=\"let submenu of link.submenu\" (click)=\"customLinkAction(submenu,column)\">\n                                <span class=\"fa-stack fa-stack-groupsort\">\n                                    <i class=\"{{submenu.icon}}\" aria-hidden=\"true\" *ngIf=\"submenu.icon\"></i>\n                                </span> {{submenu.label}}\n                            </button>\n                        </div>\n                    </div>\n                </ng-container>\n            </ng-container>\n\n            <!-- Deleting -->\n            <ng-container *ngIf=\"column.canDelete\">\n                <div class=\"dropdown-divider\"></div>\n                <button class=\"dropdown-item\" (click)=\"deleteColumn(columnIndex)\" [attr.data-qa]=\"'controls-delete'\">\n                    <span class=\"fa-stack fa-stack-groupsort\">\n                        <i class=\"fa fa-remove\" aria-hidden=\"true\"></i>\n                    </span> Delete This Column\n                </button>\n            </ng-container>\n\n    </div>\n</div>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
            },] },
];
ControlsComponent.ctorParameters = function () { return []; };
ControlsComponent.propDecorators = {
    "state": [{ type: Input },],
    "status": [{ type: Input },],
    "options": [{ type: Input },],
    "column": [{ type: Input },],
    "filterTerms": [{ type: Input },],
    "columnIndex": [{ type: Input },],
    "onStateUpdated": [{ type: Output },],
    "onCustomLinkEvent": [{ type: Output },],
};
var HeaderRowComponent = /** @class */ (function () {
    function HeaderRowComponent() {
        this.onColumnsUpdated = new EventEmitter();
        this.onStateUpdated = new EventEmitter();
        this.onCustomLinkEvent = new EventEmitter();
        this.cellContext = {};
        this.reSizing = false;
    }
    HeaderRowComponent.prototype.ngOnInit = function () { };
    HeaderRowComponent.prototype.ngOnChanges = function () {
    };
    HeaderRowComponent.prototype.stateUpdated = function (event) {
        this.onStateUpdated.emit(event);
    };
    HeaderRowComponent.prototype.customLinkEvent = function (data) {
        this.onCustomLinkEvent.emit(data);
    };
    HeaderRowComponent.prototype.trackColumn = function (_index, item) {
        return item.$$track;
    };
    HeaderRowComponent.prototype.dragDrop = function (type, columnNewPosition) {
        var isPinned = false;
        var payload = { action: 'reorder', type: type, prop: this.dragStartProp, columnIndex: columnNewPosition };
        for (var i = this.columns.length - 1; i >= 0; i--) {
            var column = this.columns[i];
            if (column.pinnedLeft) {
                isPinned = true;
            }
            column.locked = isPinned;
            column.pinnedLeft = isPinned;
        }
        this.onColumnsUpdated.emit(payload);
    };
    HeaderRowComponent.prototype.onResizeEnd = function (event, columnIndex, type) {
        var width = Math.floor(event.rectangle.width / 2) * 2;
        if (width < 44) {
            width = 44;
        }
        this.reSizing = false;
        this.onColumnsUpdated.emit({ action: 'resize', columnIndex: columnIndex, type: type, width: width });
    };
    return HeaderRowComponent;
}());
HeaderRowComponent.decorators = [
    { type: Component, args: [{
                selector: 'datagrid-header-row',
                template: "<div draggable=\"false\">\n    <div class=\"datagrid-header-cell\" [title]=\"column.label\"\n         *ngFor=\"let column of columns; let columnIndex = index; trackBy:trackColumn\"\n         [ngClass]=\"{'reorderable': column.canReorder, 'column-last' : columnIndex == columns.length - 1 }\"\n         [ngStyle]=\"{'width' : column.$$width + 'px', 'left' : column.$$leftOffset + 'px' }\"\n         mwlResizable [enableGhostResize]=\"true\" [resizeEdges]=\"{right: column.canResize || false}\" (resizeEnd)=\"onResizeEnd($event, column.$$index, columnType)\"\n         [draggable]=\"column.canReorder\" (dragstart)=\"dragStartProp = column.prop;\" (dragover)=\"$event.preventDefault()\" (drop)=\"dragDrop(columnType, column.$$index)\">\n\n        <!-- Column Label -->\n        <span class=\"datagrid-header-cell-label\">\n            <ng-template #cellTemplate\n                         *ngIf=\"templates && templates[column.prop] && templates[column.prop].templateHeader; else defaults\"\n                         [ngTemplateOutlet]=\"templates[column.prop].templateHeader\"\n                         [ngTemplateOutletContext]=\"cellContext\">\n            </ng-template>\n\n            <ng-template #defaults>\n                {{column.label}}\n            </ng-template>\n        </span>\n\n        <!-- Controls Dropdown -->\n        <datagrid-controls *ngIf=\"options.controlsDropdown\"\n                           [options]=\"options\"\n                           [status]=\"status\"\n                           [column]=\"column\"\n                           [columnIndex]=\"columnIndex\"\n                           [filterTerms]=\"filterTerms\"\n                           (onStateUpdated)=\"stateUpdated($event)\"\n                           (onCustomLinkEvent)=\"customLinkEvent($event)\">\n        </datagrid-controls>\n\n        <!-- Resize Handle -->\n        <a class=\"resize-handle\" mwlResizeHandle [resizeEdges]=\"{right: true}\" *ngIf=\"column.canResize\" (mousedown)=\"reSizing = true\"></a>\n\n    </div>\n</div>",
                styles: [],
                changeDetection: ChangeDetectionStrategy.OnPush,
            },] },
];
HeaderRowComponent.ctorParameters = function () { return []; };
HeaderRowComponent.propDecorators = {
    "columns": [{ type: Input },],
    "state": [{ type: Input },],
    "status": [{ type: Input },],
    "options": [{ type: Input },],
    "scrollProps": [{ type: Input },],
    "filterTerms": [{ type: Input },],
    "columnType": [{ type: Input },],
    "templates": [{ type: Input },],
    "onColumnsUpdated": [{ type: Output },],
    "onStateUpdated": [{ type: Output },],
    "onCustomLinkEvent": [{ type: Output },],
};
var GroupHeaderComponent = /** @class */ (function () {
    function GroupHeaderComponent() {
        this.onGroupToggled = new EventEmitter();
        this.groupLabel = '';
        this.active = true;
    }
    GroupHeaderComponent.prototype.ngOnChanges = function () {
        this.createGroupLabel(this.group, this.options);
    };
    GroupHeaderComponent.prototype.ngOnInit = function () { };
    GroupHeaderComponent.prototype.toggleGroup = function (group, $event) {
        $event.preventDefault();
        $event.stopPropagation();
        if (group && group.rows) {
            group.rows.forEach(function (row) {
                if (!row.$$hidden) {
                    row.$$hidden = true;
                }
                else {
                    row.$$hidden = false;
                }
            });
            this.group.visible = !this.group.visible;
            this.active = !this.active;
            this.onGroupToggled.emit(this.group);
        }
    };
    GroupHeaderComponent.prototype.createGroupLabel = function (group, options) {
        var _this = this;
        if (group &&
            group.columnProp &&
            options.columnData &&
            options.columnData[group.columnProp] &&
            options.columnData[group.columnProp].model) {
            this.sub = options.columnData[group.columnProp].model.subscribe(function (model) {
                if (options.columnData &&
                    group &&
                    group.label &&
                    Array.isArray(group.label) &&
                    group.columnProp &&
                    group.rows) {
                    var columnData_1 = options.columnData[group.columnProp];
                    var newLabel_1 = [];
                    group.rows[0][group.columnProp].forEach(function (id) {
                        model[columnData_1.modelSrc].forEach(function (element) {
                            if (id === element[columnData_1.value]) {
                                newLabel_1.push(element[columnData_1.label]);
                            }
                        });
                    });
                    _this.groupLabel = newLabel_1.length ? newLabel_1.join(' & ') : 'Missing Value';
                }
                else {
                    _this.groupLabel = 'No Value';
                }
            });
        }
        else {
            this.groupLabel = this.group.label ? this.group.label : 'No Value';
        }
    };
    GroupHeaderComponent.prototype.ngOnDestroy = function () {
        if (this.sub) {
            this.sub.unsubscribe();
        }
    };
    return GroupHeaderComponent;
}());
GroupHeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'group-header',
                template: "<div class=\"datagrid-group-header\">\n    <div class=\"datagrid-group-header-cell\" [ngStyle]=\"{'width' : width + 'px' }\">\n        <a (click)=\"toggleGroup(group, $event)\" [ngClass]=\"{'active': active }\">\n            <span class=\"sticky-left\">\n                <strong>\n                    <span class=\"red\" *ngIf=\"options?.showGroupColumnName\">{{group.columnLabel}}:</span>\n\n                    <ng-container *ngIf=\"group?.column?.columnType && groupLabel != 'No Value' && groupLabel != 'Missing Value' \">\n                        <ng-container [ngSwitch]=\"group.column.columnType\">\n                            <!-- Date Time-->\n                            <ng-container *ngSwitchCase=\"'dateTime'\">\n                                {{groupLabel | date: 'MM/dd/yyyy - h:mm a' }}\n                            </ng-container>\n                            <!-- Date-->\n                            <ng-container *ngSwitchCase=\"'date'\">\n                                {{groupLabel | date: 'MM/dd/yyyy' }}\n                            </ng-container>\n                            <!-- Money -->\n                            <ng-container *ngSwitchCase=\"'money'\">\n                                {{groupLabel | currency:'USD':true:'1.2-2' }}\n                            </ng-container>\n                            <!-- Default -->\n                            <ng-container *ngSwitchDefault>\n                                {{groupLabel}}\n                            </ng-container>\n                        </ng-container>\n                    </ng-container>\n                    <ng-container *ngIf=\"!group?.column?.columnType\">\n                        {{groupLabel}}\n                    </ng-container>\n                </strong> <small *ngIf=\"group?.rows?.length\"><em>({{group.rows.length}})</em></small>\n            </span>\n        </a>\n    </div>\n</div>\n\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
            },] },
];
GroupHeaderComponent.ctorParameters = function () { return []; };
GroupHeaderComponent.propDecorators = {
    "width": [{ type: Input },],
    "group": [{ type: Input },],
    "options": [{ type: Input },],
    "onGroupToggled": [{ type: Output },],
};
var RowComponent = /** @class */ (function () {
    function RowComponent(elementRef) {
        this.elementRef = elementRef;
        this.updateDatatable = new EventEmitter();
        this.onRowUpdated = new EventEmitter();
    }
    RowComponent.prototype.ngOnInit = function () { };
    RowComponent.prototype.ngOnChanges = function () { };
    RowComponent.prototype.trackColumn = function (_index, item) {
        return item.$$track;
    };
    RowComponent.prototype.onUpdateDatatable = function (action) {
        this.updateDatatable.emit(action);
    };
    RowComponent.prototype.rowUpdated = function (event) {
        this.onRowUpdated.emit(event);
    };
    return RowComponent;
}());
RowComponent.decorators = [
    { type: Component, args: [{
                selector: 'datagrid-body-row',
                template: "<div class=\"datagrid-row\">\n    <ng-container *ngFor=\"let column of columns; let columnIndex = index; trackBy:trackColumn\">\n\n        <datagrid-cell class=\"datagrid-cell\"\n                       [ngStyle]=\"{'left' : column.$$leftOffset + 'px' }\"\n                       [column]=\"column\"\n                       [row]=\"row\"\n                       [options]=\"options\"\n                       [templates]=\"templates\"\n                       (onRowUpdated)=\"rowUpdated($event)\">\n        </datagrid-cell>\n\n    </ng-container>\n</div>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
            },] },
];
RowComponent.ctorParameters = function () { return [
    { type: ElementRef, },
]; };
RowComponent.propDecorators = {
    "columns": [{ type: Input },],
    "options": [{ type: Input },],
    "row": [{ type: Input },],
    "templates": [{ type: Input },],
    "updateDatatable": [{ type: Output },],
    "onRowUpdated": [{ type: Output },],
};
var CellComponent = /** @class */ (function () {
    function CellComponent() {
        this.updateDatatable = new EventEmitter();
        this.onRowUpdated = new EventEmitter();
        this.cellContext = {};
        this.truncated = false;
    }
    CellComponent.prototype.ngOnInit = function () { };
    CellComponent.prototype.ngOnChanges = function () {
        if (this.row && this.column.prop) {
            this.checkTruncated();
            this.cellContext.column = this.column;
            this.cellContext.row = this.row;
            this.cellContext.options = this.options;
            this.cellContext.value = this.row[this.column.prop];
        }
    };
    CellComponent.prototype.ngAfterViewInit = function () {
        this.checkTruncated();
    };
    CellComponent.prototype.checkTruncated = function () {
        var _this = this;
        setTimeout(function () {
            if (_this.cellData &&
                _this.column &&
                _this.column.$$width &&
                _this.cellData.nativeElement &&
                _this.cellData.nativeElement.getBoundingClientRect().width > _this.column.$$width) {
                _this.truncated = true;
            }
            else {
                _this.truncated = false;
            }
        }, 10);
    };
    CellComponent.prototype.fieldEdit = function () {
        var _this = this;
        if (this.column.canEdit) {
            this.p.open();
            setTimeout(function () { return _this.editBox.nativeElement.focus(); });
        }
    };
    CellComponent.prototype.rowUpdated = function (event) {
        var _this = this;
        if (this.column.prop && this.row) {
            var valueOld = this.row[this.column.prop];
            this.row[this.column.prop] = event.target.value;
            var fieldData = {
                valueNew: event.target.value,
                valueOld: valueOld,
                prop: this.column.prop,
                row: this.row,
            };
            if (fieldData.valueNew !== fieldData.valueOld) {
                this.onRowUpdated.emit(fieldData);
            }
            setTimeout(function () { return _this.p.close(); });
        }
        event.preventDefault();
        event.stopPropagation();
    };
    CellComponent.prototype.onUpdateDatatable = function (action) {
        this.updateDatatable.emit(action);
    };
    CellComponent.prototype.ngOnDestroy = function () {
        if (this.cellTemplate) {
            this.cellTemplate.clear();
        }
    };
    return CellComponent;
}());
CellComponent.decorators = [
    { type: Component, args: [{
                selector: 'datagrid-cell',
                template: "<div class=\"datagrid-cell-data\"\n     title=\"{{row[column.prop]}}\"\n     [ngStyle]=\"{'width' : column.$$width + 'px'}\"\n     [ngClass]=\"{'canEdit': column.canEdit }\"\n     [ngbPopover]=\"fieldContent\"\n     #p=\"ngbPopover\"\n     container=\"body\"\n     placement=\"top-left\"\n     triggers=\"manual\"\n     (dblclick)=\"fieldEdit();\">\n\n    <div class=\"datagrid-cell-data-content\" #cellData>\n        \n        <ng-template #cellTemplate\n                     *ngIf=\"templates && templates[column.prop] && templates[column.prop].templateCell; else defaults\"\n                     [ngTemplateOutlet]=\"templates[column.prop].templateCell\"\n                     [ngTemplateOutletContext]=\"cellContext\">\n        </ng-template>\n\n        <ng-template #defaults>\n            <ng-container [ngSwitch]=\"column.columnType\">\n\n                <!-- Date Time-->\n                <ng-container *ngSwitchCase=\"'dateTime'\">\n                    {{row[column.prop] | date: 'MM/dd/yyyy - h:mm a' }}\n                </ng-container>\n\n                <!-- Date-->\n                <ng-container *ngSwitchCase=\"'date'\">\n                    {{row[column.prop] | date: 'MM/dd/yyyy' }}\n                </ng-container>\n\n                <!-- Money -->\n                <ng-container *ngSwitchCase=\"'money'\">\n                    {{row[column.prop] | currency:'USD' }}\n                </ng-container>\n\n                <!-- Default -->\n                <ng-container *ngSwitchDefault>\n                    {{row[column.prop]}}\n                </ng-container>\n\n            </ng-container>\n        </ng-template>\n\n    </div>\n\n  <div class=\"datagrid-truncated\" *ngIf=\"truncated\"></div>\n</div>\n\n<ng-template #fieldContent>\n  <textarea class=\"form-control form-control-sm\" style=\"width:400px;\" rows=\"3\" #editBox autofocus\n            (dblclick)=\"$event.target.select();\"\n            (blur)=\"rowUpdated($event);\"\n            (keyup.enter)=\"rowUpdated($event);\">{{row[column.prop]}}</textarea>\n</ng-template>\n",
                providers: [],
                changeDetection: ChangeDetectionStrategy.OnPush,
            },] },
];
CellComponent.ctorParameters = function () { return []; };
CellComponent.propDecorators = {
    "column": [{ type: Input },],
    "row": [{ type: Input },],
    "options": [{ type: Input },],
    "templates": [{ type: Input },],
    "updateDatatable": [{ type: Output },],
    "onRowUpdated": [{ type: Output },],
    "p": [{ type: ViewChild, args: ['p',] },],
    "editBox": [{ type: ViewChild, args: ['editBox',] },],
    "cellData": [{ type: ViewChild, args: ['cellData',] },],
    "cellTemplate": [{ type: ViewChild, args: ['cellTemplate', { read: ViewContainerRef },] },],
};
var InfoComponent = /** @class */ (function () {
    function InfoComponent() {
        this.onReset = new EventEmitter();
    }
    InfoComponent.prototype.ngOnInit = function () { };
    InfoComponent.prototype.reset = function (resetType) {
        this.onReset.emit(resetType);
    };
    return InfoComponent;
}());
InfoComponent.decorators = [
    { type: Component, args: [{
                selector: 'datagrid-info',
                template: "<div class=\"datagrid-info-row\" *ngIf=\"columnsMapped && state\">\n    <span *ngIf=\"state.groups.length && columnsMapped[state?.groups[0].prop]\">\n        Grouped by <strong>{{columnsMapped[state?.groups[0].prop].label}}, {{state.groups[0].dir | uppercase}}</strong>\n        <a (click)=\"reset('groups')\" *ngIf=\"options.controlsDropdown\" title=\"Remove Grouping\"><i class=\"fa fa-remove\"></i></a>\n    </span>\n    <ng-container *ngIf=\"state.groups.length && state.sorts.length\"> | </ng-container>\n    <span *ngIf=\"state.sorts.length && columnsMapped[state?.sorts[0].prop]\">\n        Sorted by <strong>{{columnsMapped[state?.sorts[0].prop].label}}, {{state.sorts[0].dir | uppercase}}</strong>\n        <a (click)=\"reset('sorts')\" *ngIf=\"options.controlsDropdown\" title=\"Remove Sorting\"><i class=\"fa fa-remove\"></i></a>\n    </span>\n    <ng-container *ngIf=\"state.sorts.length && state.filters.length\"> | </ng-container>\n    <span *ngIf=\"state.filters.length\">\n        <strong>{{state?.filters?.length}} Filter/s Active</strong> {{state?.info?.rowsVisible}}/{{state?.info?.rowsTotal}} Rows Visible:\n        <small *ngFor=\"let filter of state.filters\">\n            ({{columnsMapped[filter.prop].label}}\n            <ng-container *ngIf=\"filter.operator == 'eq'\">=</ng-container>\n            <ng-container *ngIf=\"filter.operator == 'contains'\">contains</ng-container>\n            {{filter.value}})\n        </small>\n        <a (click)=\"reset('filters')\" *ngIf=\"options.controlsDropdown\" title=\"Remove All Filters\"><i class=\"fa fa-remove\"></i></a>\n    </span>\n    <span *ngIf=\"options?.filterGlobal?.term\"> | Global Filter: <strong>{{options.filterGlobal.term}}</strong></span>\n</div>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
            },] },
];
InfoComponent.ctorParameters = function () { return []; };
InfoComponent.propDecorators = {
    "state": [{ type: Input },],
    "options": [{ type: Input },],
    "columnsMapped": [{ type: Input },],
    "onReset": [{ type: Output },],
};
var DatagridModule = /** @class */ (function () {
    function DatagridModule() {
    }
    DatagridModule.forRoot = function () {
        return {
            ngModule: DatagridModule,
            providers: [DataGridService],
        };
    };
    return DatagridModule;
}());
DatagridModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, NgbModule.forRoot(), FormsModule, ResizableModule],
                declarations: [
                    DataGridComponent,
                    RowComponent,
                    GroupHeaderComponent,
                    HeaderComponent,
                    ControlsComponent,
                    FiltersComponent,
                    BodyComponent,
                    CellComponent,
                    InfoComponent,
                    HeaderRowComponent,
                    DataTableColumnDirective,
                    DataGridColumnCellDirective,
                    DataGridColumnHeaderDirective,
                ],
                providers: [DataGridService],
                exports: [DataGridComponent, DataTableColumnDirective, DataGridColumnCellDirective, DataGridColumnHeaderDirective],
            },] },
];

export { DatagridModule, BodyComponent as ɵk, CellComponent as ɵl, GroupHeaderComponent as ɵg, RowComponent as ɵf, DataGridComponent as ɵa, ControlsComponent as ɵi, FiltersComponent as ɵj, HeaderComponent as ɵh, HeaderRowComponent as ɵn, InfoComponent as ɵm, DataGridColumnCellDirective as ɵc, DataGridColumnHeaderDirective as ɵd, DataTableColumnDirective as ɵb, DataGridService as ɵe };
//# sourceMappingURL=mello-labs-datagrid.js.map
